@@ -1,4 +1,6 @@
-{-# LANGUAGE NoMonomorphismRestriction, OverloadedStrings #-}
+{-# LANGUAGE NoMonomorphismRestriction #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
 {-|
 
 'AccountName's are strings like @assets:cash:petty@, with multiple
@@ -10,7 +12,9 @@ hierarchy.
 module Hledger.Data.AccountName
 where
 import Data.List
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
+#endif
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Tree
@@ -43,7 +47,7 @@ accountLeafName = last . accountNameComponents
 accountSummarisedName :: AccountName -> Text
 accountSummarisedName a
   --   length cs > 1 = take 2 (head cs) ++ ":" ++ a'
-  | length cs > 1 = (T.intercalate ":" (map (T.take 2) $ init cs)) <> ":" <> a'
+  | length cs > 1 = T.intercalate ":" (map (T.take 2) $ init cs) <> ":" <> a'
   | otherwise     = a'
     where
       cs = accountNameComponents a
@@ -53,8 +57,24 @@ accountNameLevel :: AccountName -> Int
 accountNameLevel "" = 0
 accountNameLevel a = T.length (T.filter (==acctsepchar) a) + 1
 
+-- | A top-level account prefixed to some accounts in budget reports.
+-- Defined here so it can be ignored by accountNameDrop. 
+unbudgetedAccountName :: T.Text
+unbudgetedAccountName = "<unbudgeted>"
+
+-- | Remove some number of account name components from the front of the account name.
+-- If the special "<unbudgeted>" top-level account is present, it is preserved and
+-- dropping affects the rest of the account name. 
 accountNameDrop :: Int -> AccountName -> AccountName
-accountNameDrop n = accountNameFromComponents . drop n . accountNameComponents
+accountNameDrop n a
+  | a == unbudgetedAccountName = a
+  | unbudgetedAccountAndSep `T.isPrefixOf` a =
+      case accountNameDrop n $ T.drop (T.length unbudgetedAccountAndSep) a of
+        "" -> unbudgetedAccountName
+        a' -> unbudgetedAccountAndSep <> a'
+  | otherwise = accountNameFromComponents $ drop n $ accountNameComponents a
+  where 
+    unbudgetedAccountAndSep = unbudgetedAccountName <> acctsep
 
 -- | Sorted unique account names implied by these account names,
 -- ie these plus all their parent accounts up to the root.
@@ -125,11 +145,11 @@ elideAccountName width s
   | " (split)" `T.isSuffixOf` s =
     let
       names = T.splitOn ", " $ T.take (T.length s - 8) s
-      widthpername = (max 0 (width - 8 - 2 * (max 1 (length names) - 1))) `div` length names
+      widthpername = max 0 (width - 8 - 2 * (max 1 (length names) - 1)) `div` length names
     in
      fitText Nothing (Just width) True False $
      (<>" (split)") $
-     T.intercalate ", " $
+     T.intercalate ", "
      [accountNameFromComponents $ elideparts widthpername [] $ accountNameComponents s' | s' <- names]
   | otherwise =
     fitText Nothing (Just width) True False $ accountNameFromComponents $ elideparts width [] $ accountNameComponents s
@@ -174,7 +194,7 @@ accountRegexToAccountName = T.pack . regexReplace "^\\^(.*?)\\(:\\|\\$\\)$" "\\1
 
 -- | Does this string look like an exact account-matching regular expression ?
 isAccountRegex  :: String -> Bool
-isAccountRegex s = take 1 s == "^" && (take 5 $ reverse s) == ")$|:("
+isAccountRegex s = take 1 s == "^" && take 5 (reverse s) == ")$|:("
 
 tests_Hledger_Data_AccountName = TestList
  [
