@@ -8,12 +8,16 @@ are thousands separated by comma, significant decimal places and so on.
 -}
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
 
 module Hledger.Data.Commodity
 where
+import Data.Char (isDigit)
 import Data.List
 import Data.Maybe (fromMaybe)
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
+#endif
 import qualified Data.Text as T
 import Test.HUnit
 -- import qualified Data.Map as M
@@ -25,7 +29,13 @@ import Hledger.Utils
 -- characters that may not be used in a non-quoted commodity symbol
 nonsimplecommoditychars = "0123456789-+.@*;\n \"{}=" :: [Char]
 
-quoteCommoditySymbolIfNeeded s | any (`elem` nonsimplecommoditychars) (T.unpack s) = "\"" <> s <> "\""
+isNonsimpleCommodityChar :: Char -> Bool
+isNonsimpleCommodityChar c = isDigit c || c `textElem` otherChars
+ where
+   otherChars = "-+.@*;\n \"{}=" :: T.Text
+   textElem = T.any . (==)
+
+quoteCommoditySymbolIfNeeded s | T.any (isNonsimpleCommodityChar) s = "\"" <> s <> "\""
                                | otherwise = s
 
 commodity = ""
