@@ -4,6 +4,7 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE PackageImports #-}
 {-|
 
 Date parsing and utilities for hledger.
@@ -40,20 +41,25 @@ module Hledger.Data.Dates (
   parsedate,
   showDate,
   showDateSpan,
+  showDateSpanMonthAbbrev,
   elapsedSeconds,
   prevday,
+  periodexprp,
   parsePeriodExpr,
+  parsePeriodExpr',
   nulldatespan,
   failIfInvalidYear,
   failIfInvalidMonth,
   failIfInvalidDay,
   datesepchar,
   datesepchars,
+  isDateSepChar,
   spanStart,
   spanEnd,
   spansSpan,
   spanIntersect,
   spansIntersect,
+  spanIntervalIntersect,
   spanDefaultsFrom,
   spanUnion,
   spansUnion,
@@ -70,9 +76,9 @@ module Hledger.Data.Dates (
 where
 
 import Prelude ()
-import Prelude.Compat
+import "base-compat-batteries" Prelude.Compat
 import Control.Monad
-import Data.List.Compat
+import "base-compat-batteries" Data.List.Compat
 import Data.Default
 import Data.Maybe
 import Data.Text (Text)
@@ -88,7 +94,8 @@ import Data.Time.Calendar.OrdinalDate
 import Data.Time.Clock
 import Data.Time.LocalTime
 import Safe (headMay, lastMay, readMay)
-import Text.Megaparsec.Compat
+import Text.Megaparsec
+import Text.Megaparsec.Char
 import Text.Megaparsec.Perm
 import Text.Printf
 
@@ -110,6 +117,11 @@ showDate = formatTime defaultTimeLocale "%0C%y/%m/%d"
 showDateSpan :: DateSpan -> String
 showDateSpan = showPeriod . dateSpanAsPeriod
 
+-- | Like showDateSpan, but show month spans as just the abbreviated month name
+-- in the current locale.
+showDateSpanMonthAbbrev :: DateSpan -> String
+showDateSpanMonthAbbrev = showPeriodMonthAbbrev . dateSpanAsPeriod
+
 -- | Get the current local date.
 getCurrentDay :: IO Day
 getCurrentDay = do
@@ -143,8 +155,12 @@ spanEnd (DateSpan _ d) = d
 spansSpan :: [DateSpan] -> DateSpan
 spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Nothing spanEnd $ lastMay spans)
 
--- | Split a DateSpan into one or more consecutive whole spans of the specified length which enclose it.
+-- | Split a DateSpan into consecutive whole spans of the specified interval
+-- which fully encompass the original span (and a little more when necessary).
 -- If no interval is specified, the original span is returned.
+-- If the original span is the null date span, ie unbounded, the null date span is returned.
+-- If the original span is empty, eg if the end date is <= the start date, no spans are returned.
+-- 
 --
 -- ==== Examples:
 -- >>> let t i d1 d2 = splitSpan i $ mkdatespan d1 d2
@@ -155,9 +171,9 @@ spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Noth
 -- >>> splitSpan (Quarters 1) nulldatespan
 -- [DateSpan -]
 -- >>> t (Days 1) "2008/01/01" "2008/01/01"  -- an empty datespan
--- [DateSpan 2008/01/01-2007/12/31]
+-- []
 -- >>> t (Quarters 1) "2008/01/01" "2008/01/01"
--- [DateSpan 2008/01/01-2007/12/31]
+-- []
 -- >>> t (Months 1) "2008/01/01" "2008/04/01"
 -- [DateSpan 2008/01,DateSpan 2008/02,DateSpan 2008/03]
 -- >>> t (Months 2) "2008/01/01" "2008/04/01"
@@ -179,6 +195,7 @@ spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Noth
 --
 splitSpan :: Interval -> DateSpan -> [DateSpan]
 splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
+splitSpan _ s | isEmptySpan s = []
 splitSpan NoInterval     s = [s]
 splitSpan (Days n)       s = splitspan startofday     (applyN n nextday)     s
 splitSpan (Weeks n)      s = splitspan startofweek    (applyN n nextweek)    s
@@ -216,6 +233,12 @@ daysInSpan :: DateSpan -> Maybe Integer
 daysInSpan (DateSpan (Just d1) (Just d2)) = Just $ diffDays d2 d1
 daysInSpan _ = Nothing
 
+-- | Is this an empty span, ie closed with the end date on or before the start date ?
+isEmptySpan :: DateSpan -> Bool
+isEmptySpan s = case daysInSpan s of
+                  Just n  -> n < 1
+                  Nothing -> False
+
 -- | Does the span include the given date ?
 spanContainsDate :: DateSpan -> Day -> Bool
 spanContainsDate (DateSpan Nothing Nothing)   _ = True
@@ -234,11 +257,36 @@ spansIntersect [d] = d
 spansIntersect (d:ds) = d `spanIntersect` (spansIntersect ds)
 
 -- | Calculate the intersection of two datespans.
+--
+-- For non-intersecting spans, gives an empty span beginning on the second's start date:
+-- >>> mkdatespan "2018-01-01" "2018-01-03" `spanIntersect` mkdatespan "2018-01-03" "2018-01-05"
+-- DateSpan 2018/01/03-2018/01/02
 spanIntersect (DateSpan b1 e1) (DateSpan b2 e2) = DateSpan b e
     where
       b = latest b1 b2
       e = earliest e1 e2
 
+-- | Calculate the intersection of two DateSpans, adjusting the start date so
+-- the interval is preserved.
+--
+-- >>> let intervalIntersect = spanIntervalIntersect (Days 3)
+-- >>> mkdatespan "2018-01-01" "2018-01-03" `intervalIntersect` mkdatespan "2018-01-01" "2018-01-05"
+-- DateSpan 2018/01/01-2018/01/02
+-- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-02" "2018-01-05"
+-- DateSpan 2018/01/04
+-- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-03" "2018-01-05"
+-- DateSpan 2018/01/04
+-- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-04" "2018-01-05"
+-- DateSpan 2018/01/04
+-- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2017-12-01" "2018-01-05"
+-- DateSpan 2018/01/01-2018/01/04
+spanIntervalIntersect :: Interval -> DateSpan -> DateSpan -> DateSpan
+spanIntervalIntersect (Days n) (DateSpan (Just b1) e1) sp2@(DateSpan (Just b2) _) =
+      DateSpan (Just b) e1 `spanIntersect` sp2
+    where
+      b = if b1 < b2 then addDays (diffDays b1 b2 `mod` toInteger n) b2 else b1
+spanIntervalIntersect _ sp1 sp2 = sp1 `spanIntersect` sp2
+
 -- | Fill any unspecified dates in the first span with the dates from
 -- the second one. Sort of a one-way spanIntersect.
 spanDefaultsFrom (DateSpan a1 b1) (DateSpan a2 b2) = DateSpan a b
@@ -266,8 +314,14 @@ earliest (Just d1) (Just d2) = Just $ min d1 d2
 
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
-parsePeriodExpr :: Day -> Text -> Either (ParseError Char MPErr) (Interval, DateSpan)
-parsePeriodExpr refdate s = parsewith (periodexpr refdate <* eof) (T.toLower s)
+parsePeriodExpr :: Day -> Text -> Either (ParseError Char CustomErr) (Interval, DateSpan)
+parsePeriodExpr refdate s = parsewith (periodexprp refdate <* eof) (T.toLower s)
+
+-- | Like parsePeriodExpr, but call error' on failure.
+parsePeriodExpr' :: Day -> Text -> (Interval, DateSpan)
+parsePeriodExpr' refdate s =
+  either (error' . ("failed to parse:" ++) . parseErrorPretty) id $
+  parsePeriodExpr refdate s
 
 maybePeriod :: Day -> Text -> Maybe (Interval,DateSpan)
 maybePeriod refdate = either (const Nothing) Just . parsePeriodExpr refdate
@@ -326,13 +380,13 @@ fixSmartDateStr :: Day -> Text -> String
 fixSmartDateStr d s = either
                        (\e->error' $ printf "could not parse date %s %s" (show s) (show e))
                        id
-                       $ (fixSmartDateStrEither d s :: Either (ParseError Char MPErr) String)
+                       $ (fixSmartDateStrEither d s :: Either (ParseError Char CustomErr) String)
 
 -- | A safe version of fixSmartDateStr.
-fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char MPErr) String
+fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char CustomErr) String
 fixSmartDateStrEither d = either Left (Right . showDate) . fixSmartDateStrEither' d
 
-fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char MPErr) Day
+fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char CustomErr) Day
 fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
                                Right sd -> Right $ fixSmartDate d sd
                                Left e -> Left e
@@ -629,39 +683,76 @@ parsedate s =  fromMaybe (error' $ "could not parse date \"" ++ s ++ "\"")
 -- #endif
 
 {-|
-Parse a date in any of the formats allowed in ledger's period expressions,
-and maybe some others:
-
-> 2004
-> 2004/10
-> 2004/10/1
-> 10/1
-> 21
-> october, oct
-> yesterday, today, tomorrow
-> this/next/last week/day/month/quarter/year
-
-Returns a SmartDate, to be converted to a full date later (see fixSmartDate).
+Parse a date in any of the formats allowed in Ledger's period expressions, and some others.
 Assumes any text in the parse stream has been lowercased.
+Returns a SmartDate, to be converted to a full date later (see fixSmartDate).
+
+Examples:
+
+> 2004                                        (start of year, which must have 4+ digits)
+> 2004/10                                     (start of month, which must be 1-12)
+> 2004/10/1                                   (exact date, day must be 1-31)
+> 10/1                                        (month and day in current year)
+> 21                                          (day in current month)
+> october, oct                                (start of month in current year)
+> yesterday, today, tomorrow                  (-1, 0, 1 days from today)
+> last/this/next day/week/month/quarter/year  (-1, 0, 1 periods from the current period)
+> 20181201                                    (8 digit YYYYMMDD with valid year month and day)
+> 201812                                      (6 digit YYYYMM with valid year and month)
+
+Note malformed digit sequences might give surprising results:
+
+> 201813                                      (6 digits with an invalid month is parsed as start of 6-digit year)
+> 20181301                                    (8 digits with an invalid month is parsed as start of 8-digit year)
+> 20181232                                    (8 digits with an invalid day gives an error)
+> 201801012                                   (9+ digits beginning with a valid YYYYMMDD gives an error)
+
+Eg:
+
+YYYYMMDD is parsed as year-month-date if those parts are valid
+(>=4 digits, 1-12, and 1-31 respectively):
+>>> parsewith (smartdate <* eof) "20181201"
+Right ("2018","12","01")
+
+YYYYMM is parsed as year-month-01 if year and month are valid:
+>>> parsewith (smartdate <* eof) "201804"
+Right ("2018","04","01")
+
+With an invalid month, it's parsed as a year:
+>>> parsewith (smartdate <* eof) "201813"
+Right ("201813","","")
+
+A 9+ digit number beginning with valid YYYYMMDD gives an error:
+>>> parsewith (smartdate <* eof) "201801012"
+Left (...)
+
+Big numbers not beginning with a valid YYYYMMDD are parsed as a year:
+>>> parsewith (smartdate <* eof) "201813012"
+Right ("201813012","","")
+
 -}
-smartdate :: SimpleTextParser SmartDate
+smartdate :: TextParser m SmartDate
 smartdate = do
   -- XXX maybe obscures date errors ? see ledgerdate
-  (y,m,d) <- choice' [yyyymmdd, ymd, ym, md, y, d, month, mon, today, yesterday, tomorrow, lastthisnextthing]
+  (y,m,d) <- choice' [yyyymmdd, yyyymm, ymd, ym, md, y, d, month, mon, today, yesterday, tomorrow, lastthisnextthing]
   return (y,m,d)
 
 -- | Like smartdate, but there must be nothing other than whitespace after the date.
-smartdateonly :: SimpleTextParser SmartDate
+smartdateonly :: TextParser m SmartDate
 smartdateonly = do
   d <- smartdate
-  many spacenonewline
+  skipMany spacenonewline
   eof
   return d
 
 datesepchars :: [Char]
 datesepchars = "/-."
+
 datesepchar :: TextParser m Char
-datesepchar = oneOf datesepchars
+datesepchar = satisfy isDateSepChar
+
+isDateSepChar :: Char -> Bool
+isDateSepChar c = c == '/' || c == '-' || c == '.'
 
 validYear, validMonth, validDay :: String -> Bool
 validYear s = length s >= 4 && isJust (readMay s :: Maybe Year)
@@ -673,7 +764,7 @@ failIfInvalidYear s  = unless (validYear s)  $ fail $ "bad year number: " ++ s
 failIfInvalidMonth s = unless (validMonth s) $ fail $ "bad month number: " ++ s
 failIfInvalidDay s   = unless (validDay s)   $ fail $ "bad day number: " ++ s
 
-yyyymmdd :: SimpleTextParser SmartDate
+yyyymmdd :: TextParser m SmartDate
 yyyymmdd = do
   y <- count 4 digitChar
   m <- count 2 digitChar
@@ -682,7 +773,14 @@ yyyymmdd = do
   failIfInvalidDay d
   return (y,m,d)
 
-ymd :: SimpleTextParser SmartDate
+yyyymm :: TextParser m SmartDate
+yyyymm = do
+  y <- count 4 digitChar
+  m <- count 2 digitChar
+  failIfInvalidMonth m
+  return (y,m,"01")
+
+ymd :: TextParser m SmartDate
 ymd = do
   y <- some digitChar
   failIfInvalidYear y
@@ -694,7 +792,7 @@ ymd = do
   failIfInvalidDay d
   return $ (y,m,d)
 
-ym :: SimpleTextParser SmartDate
+ym :: TextParser m SmartDate
 ym = do
   y <- some digitChar
   failIfInvalidYear y
@@ -703,19 +801,19 @@ ym = do
   failIfInvalidMonth m
   return (y,m,"")
 
-y :: SimpleTextParser SmartDate
+y :: TextParser m SmartDate
 y = do
   y <- some digitChar
   failIfInvalidYear y
   return (y,"","")
 
-d :: SimpleTextParser SmartDate
+d :: TextParser m SmartDate
 d = do
   d <- some digitChar
   failIfInvalidDay d
   return ("","",d)
 
-md :: SimpleTextParser SmartDate
+md :: TextParser m SmartDate
 md = do
   m <- some digitChar
   failIfInvalidMonth m
@@ -733,38 +831,38 @@ weekdayabbrevs = ["mon","tue","wed","thu","fri","sat","sun"]
 monthIndex t = maybe 0 (+1) $ t `elemIndex` months
 monIndex t   = maybe 0 (+1) $ t `elemIndex` monthabbrevs
 
-month :: SimpleTextParser SmartDate
+month :: TextParser m SmartDate
 month = do
-  m <- choice $ map (try . string) months
+  m <- choice $ map (try . string') months
   let i = monthIndex m
   return ("",show i,"")
 
-mon :: SimpleTextParser SmartDate
+mon :: TextParser m SmartDate
 mon = do
-  m <- choice $ map (try . string) monthabbrevs
+  m <- choice $ map (try . string') monthabbrevs
   let i = monIndex m
   return ("",show i,"")
 
-weekday :: SimpleTextParser Int
+weekday :: TextParser m Int
 weekday = do
   wday <- choice . map string' $ weekdays ++ weekdayabbrevs
   let i = head . catMaybes $ [wday `elemIndex` weekdays, wday `elemIndex` weekdayabbrevs]
   return (i+1)
 
-today,yesterday,tomorrow :: SimpleTextParser SmartDate
-today     = string "today"     >> return ("","","today")
-yesterday = string "yesterday" >> return ("","","yesterday")
-tomorrow  = string "tomorrow"  >> return ("","","tomorrow")
+today,yesterday,tomorrow :: TextParser m SmartDate
+today     = string' "today"     >> return ("","","today")
+yesterday = string' "yesterday" >> return ("","","yesterday")
+tomorrow  = string' "tomorrow"  >> return ("","","tomorrow")
 
-lastthisnextthing :: SimpleTextParser SmartDate
+lastthisnextthing :: TextParser m SmartDate
 lastthisnextthing = do
-  r <- choice $ map mptext [
+  r <- choice $ map string' [
         "last"
        ,"this"
        ,"next"
       ]
-  many spacenonewline  -- make the space optional for easier scripting
-  p <- choice $ map mptext [
+  skipMany spacenonewline  -- make the space optional for easier scripting
+  p <- choice $ map string' [
         "day"
        ,"week"
        ,"month"
@@ -772,7 +870,7 @@ lastthisnextthing = do
        ,"year"
       ]
 -- XXX support these in fixSmartDate
---       ++ (map string $ months ++ monthabbrevs ++ weekdays ++ weekdayabbrevs)
+--       ++ (map string' $ months ++ monthabbrevs ++ weekdays ++ weekdayabbrevs)
 
   return ("", T.unpack r, T.unpack p)
 
@@ -820,96 +918,98 @@ lastthisnextthing = do
 -- Right (DayOfMonth 2,DateSpan 2009/01/01-)
 -- >>> p "every 2nd day of month 2009-"
 -- Right (DayOfMonth 2,DateSpan 2009/01/01-)
-periodexpr :: Day -> SimpleTextParser (Interval, DateSpan)
-periodexpr rdate = surroundedBy (many spacenonewline) . choice $ map try [
-                    intervalanddateperiodexpr rdate,
-                    (,) NoInterval <$> periodexprdatespan rdate
+periodexprp :: Day -> TextParser m (Interval, DateSpan)
+periodexprp rdate = do
+  skipMany spacenonewline
+  choice $ map try [
+                    intervalanddateperiodexprp rdate,
+                    (,) NoInterval <$> periodexprdatespanp rdate
                    ]
 
-intervalanddateperiodexpr :: Day -> SimpleTextParser (Interval, DateSpan)
-intervalanddateperiodexpr rdate = do
-  i <- reportinginterval
+intervalanddateperiodexprp :: Day -> TextParser m (Interval, DateSpan)
+intervalanddateperiodexprp rdate = do
+  i <- reportingintervalp
   s <- option def . try $ do
-      many spacenonewline
-      periodexprdatespan rdate
+      skipMany spacenonewline
+      periodexprdatespanp rdate
   return (i,s)
 
 -- Parse a reporting interval.
-reportinginterval :: SimpleTextParser Interval
-reportinginterval = choice' [
+reportingintervalp :: TextParser m Interval
+reportingintervalp = choice' [
                        tryinterval "day"     "daily"     Days,
                        tryinterval "week"    "weekly"    Weeks,
                        tryinterval "month"   "monthly"   Months,
                        tryinterval "quarter" "quarterly" Quarters,
                        tryinterval "year"    "yearly"    Years,
-                       do string "biweekly"
+                       do string' "biweekly"
                           return $ Weeks 2,
-                       do string "bimonthly"
+                       do string' "bimonthly"
                           return $ Months 2,
-                       do string "every"
-                          many spacenonewline
+                       do string' "every"
+                          skipMany spacenonewline
                           n <- nth
-                          many spacenonewline
-                          string "day"
+                          skipMany spacenonewline
+                          string' "day"
                           of_ "week"
                           return $ DayOfWeek n,
-                       do string "every"
-                          many spacenonewline
+                       do string' "every"
+                          skipMany spacenonewline
                           n <- weekday
                           return $ DayOfWeek n,
-                       do string "every"
-                          many spacenonewline
+                       do string' "every"
+                          skipMany spacenonewline
                           n <- nth
-                          many spacenonewline
-                          string "day"
+                          skipMany spacenonewline
+                          string' "day"
                           optOf_ "month"
                           return $ DayOfMonth n,
-                       do string "every"
+                       do string' "every"
                           let mnth = choice' [month, mon] >>= \(_,m,_) -> return (read m)
-                          d_o_y <- makePermParser $ DayOfYear <$$> try (many spacenonewline *> mnth) <||> try (many spacenonewline *> nth)
+                          d_o_y <- makePermParser $ DayOfYear <$$> try (skipMany spacenonewline *> mnth) <||> try (skipMany spacenonewline *> nth)
                           optOf_ "year"
                           return d_o_y,
-                       do string "every"
-                          many spacenonewline
+                       do string' "every"
+                          skipMany spacenonewline
                           ("",m,d) <- md
                           optOf_ "year"
                           return $ DayOfYear (read m) (read d),
-                       do string "every"
-                          many spacenonewline
+                       do string' "every"
+                          skipMany spacenonewline
                           n <- nth
-                          many spacenonewline
+                          skipMany spacenonewline
                           wd <- weekday
                           optOf_ "month"
                           return $ WeekdayOfMonth n wd
                     ]
     where
       of_ period = do
-        many spacenonewline
-        string "of"
-        many spacenonewline
-        string period
+        skipMany spacenonewline
+        string' "of"
+        skipMany spacenonewline
+        string' period
         
       optOf_ period = optional $ try $ of_ period
       
       nth = do n <- some digitChar
-               choice' $ map string ["st","nd","rd","th"]
+               choice' $ map string' ["st","nd","rd","th"]
                return $ read n
 
       -- Parse any of several variants of a basic interval, eg "daily", "every day", "every N days".
-      tryinterval :: String -> String -> (Int -> Interval) -> SimpleTextParser Interval
+      tryinterval :: String -> String -> (Int -> Interval) -> TextParser m Interval
       tryinterval singular compact intcons =
         choice' [
-          do mptext compact'
+          do string' compact'
              return $ intcons 1,
-          do mptext "every"
-             many spacenonewline
-             mptext singular'
+          do string' "every"
+             skipMany spacenonewline
+             string' singular'
              return $ intcons 1,
-          do mptext "every"
-             many spacenonewline
+          do string' "every"
+             skipMany spacenonewline
              n <- fmap read $ some digitChar
-             many spacenonewline
-             mptext plural'
+             skipMany spacenonewline
+             string' plural'
              return $ intcons n
           ]
         where
@@ -917,46 +1017,49 @@ reportinginterval = choice' [
           singular' = T.pack singular
           plural'   = T.pack $ singular ++ "s"
 
-periodexprdatespan :: Day -> SimpleTextParser DateSpan
-periodexprdatespan rdate = choice $ map try [
-                            doubledatespan rdate,
-                            fromdatespan rdate,
-                            todatespan rdate,
-                            justdatespan rdate
+periodexprdatespanp :: Day -> TextParser m DateSpan
+periodexprdatespanp rdate = choice $ map try [
+                            doubledatespanp rdate,
+                            fromdatespanp rdate,
+                            todatespanp rdate,
+                            justdatespanp rdate
                            ]
 
-doubledatespan :: Day -> SimpleTextParser DateSpan
-doubledatespan rdate = do
-  optional (string "from" >> many spacenonewline)
+-- |
+-- -- >>> parsewith (doubledatespan (parsedate "2018/01/01") <* eof) "20180101-201804"
+-- Right DateSpan 2018/01/01-2018/04/01
+doubledatespanp :: Day -> TextParser m DateSpan
+doubledatespanp rdate = do
+  optional (string' "from" >> skipMany spacenonewline)
   b <- smartdate
-  many spacenonewline
-  optional (choice [string "to", string "-"] >> many spacenonewline)
+  skipMany spacenonewline
+  optional (choice [string' "to", string' "-"] >> skipMany spacenonewline)
   e <- smartdate
   return $ DateSpan (Just $ fixSmartDate rdate b) (Just $ fixSmartDate rdate e)
 
-fromdatespan :: Day -> SimpleTextParser DateSpan
-fromdatespan rdate = do
+fromdatespanp :: Day -> TextParser m DateSpan
+fromdatespanp rdate = do
   b <- choice [
     do
-      string "from" >> many spacenonewline
+      string' "from" >> skipMany spacenonewline
       smartdate
     ,
     do
       d <- smartdate
-      string "-"
+      string' "-"
       return d
     ]
   return $ DateSpan (Just $ fixSmartDate rdate b) Nothing
 
-todatespan :: Day -> SimpleTextParser DateSpan
-todatespan rdate = do
-  choice [string "to", string "-"] >> many spacenonewline
+todatespanp :: Day -> TextParser m DateSpan
+todatespanp rdate = do
+  choice [string' "to", string' "-"] >> skipMany spacenonewline
   e <- smartdate
   return $ DateSpan Nothing (Just $ fixSmartDate rdate e)
 
-justdatespan :: Day -> SimpleTextParser DateSpan
-justdatespan rdate = do
-  optional (string "in" >> many spacenonewline)
+justdatespanp :: Day -> TextParser m DateSpan
+justdatespanp rdate = do
+  optional (string' "in" >> skipMany spacenonewline)
   d <- smartdate
   return $ spanFromSmartDate rdate d
 
