@@ -8,6 +8,7 @@ look up the date or description there.
 -}
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
 
 module Hledger.Data.Posting (
   -- * Posting
@@ -59,7 +60,9 @@ where
 import Data.List
 import Data.Maybe
 import Data.MemoUgly (memo)
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
+#endif
 import Data.Ord
 import Data.Text (Text)
 import qualified Data.Text as T
