@@ -6,6 +6,7 @@ transactions..)  by various criteria, and a SimpleTextParser for query expressio
 -}
 
 {-# LANGUAGE DeriveDataTypeable, OverloadedStrings, ViewPatterns #-}
+{-# LANGUAGE CPP #-}
 
 module Hledger.Query (
   -- * Query and QueryOpt
@@ -49,13 +50,16 @@ import Data.Data
 import Data.Either
 import Data.List
 import Data.Maybe
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid ((<>))
+#endif
 -- import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
 import Safe (readDef, headDef)
 import Test.HUnit
-import Text.Megaparsec.Compat
+import Text.Megaparsec
+import Text.Megaparsec.Char
 
 import Hledger.Utils hiding (words')
 import Hledger.Data.Types
@@ -185,13 +189,13 @@ words'' :: [T.Text] -> T.Text -> [T.Text]
 words'' prefixes = fromparse . parsewith maybeprefixedquotedphrases -- XXX
     where
       maybeprefixedquotedphrases :: SimpleTextParser [T.Text]
-      maybeprefixedquotedphrases = choice' [prefixedQuotedPattern, singleQuotedPattern, doubleQuotedPattern, pattern] `sepBy` some spacenonewline
+      maybeprefixedquotedphrases = choice' [prefixedQuotedPattern, singleQuotedPattern, doubleQuotedPattern, pattern] `sepBy` skipSome spacenonewline
       prefixedQuotedPattern :: SimpleTextParser T.Text
       prefixedQuotedPattern = do
-        not' <- fromMaybe "" `fmap` (optional $ mptext "not:")
+        not' <- fromMaybe "" `fmap` (optional $ string "not:")
         let allowednexts | T.null not' = prefixes
                          | otherwise   = prefixes ++ [""]
-        next <- choice' $ map mptext allowednexts
+        next <- choice' $ map string allowednexts
         let prefix :: T.Text
             prefix = not' <> next
         p <- singleQuotedPattern <|> doubleQuotedPattern
@@ -303,6 +307,7 @@ tests_parseQueryTerm = [
     "real:1" `gives` (Left $ Real True)
     "date:2008" `gives` (Left $ Date $ DateSpan (Just $ parsedate "2008/01/01") (Just $ parsedate "2009/01/01"))
     "date:from 2012/5/17" `gives` (Left $ Date $ DateSpan (Just $ parsedate "2012/05/17") Nothing)
+    "date:20180101-201804" `gives` (Left $ Date $ DateSpan (Just $ parsedate "2018/01/01") (Just $ parsedate "2018/04/01"))
     "inacct:a" `gives` (Right $ QueryOptInAcct "a")
     "tag:a" `gives` (Left $ Tag "a" Nothing)
     "tag:a=some value" `gives` (Left $ Tag "a" (Just "some value"))
