@@ -14,7 +14,6 @@ module Hledger.Read (
   PrefixedFilePath,
   defaultJournal,
   defaultJournalPath,
-  readJournalFilesWithOpts,
   readJournalFiles,
   readJournalFile,
   requireJournalFileExists,
@@ -26,7 +25,6 @@ module Hledger.Read (
   readJournal',
 
   -- * Re-exported
-  JournalReader.accountaliasp,
   JournalReader.postingp,
   module Hledger.Read.Common,
 
@@ -36,10 +34,10 @@ module Hledger.Read (
 
 ) where
 
-import Control.Applicative ((<|>))
 import Control.Arrow (right)
 import qualified Control.Exception as C
 import Control.Monad.Except
+import Data.Default
 import Data.List
 import Data.Maybe
 import Data.Ord
@@ -90,7 +88,7 @@ type PrefixedFilePath = FilePath
 
 -- | Read the default journal file specified by the environment, or raise an error.
 defaultJournal :: IO Journal
-defaultJournal = defaultJournalPath >>= readJournalFile Nothing Nothing True >>= either error' return
+defaultJournal = defaultJournalPath >>= readJournalFile def >>= either error' return
 
 -- | Get the default journal file path specified by the environment.
 -- Like ledger, we look first for the LEDGER_FILE environment
@@ -112,52 +110,6 @@ defaultJournalPath = do
                   home <- getHomeDirectory `C.catch` (\(_::C.IOException) -> return "")
                   return $ home </> journalDefaultFilename
 
--- | @readJournalFiles mformat mrulesfile assrt prefixedfiles@
---
--- Read a Journal from each specified file path and combine them into one.
--- Or, return the first error message.
---
--- Combining Journals means concatenating them, basically.
--- The parse state resets at the start of each file, which means that
--- directives & aliases do not cross file boundaries.
--- (The final parse state saved in the Journal does span all files, however.)
---
--- As with readJournalFile,
--- file paths can optionally have a READER: prefix,
--- and the @mformat@, @mrulesfile, and @assrt@ arguments are supported
--- (and these are applied to all files).
---
-readJournalFiles :: Maybe StorageFormat -> Maybe FilePath -> Bool -> [PrefixedFilePath] -> IO (Either String Journal)
-readJournalFiles mformat mrulesfile assrt prefixedfiles = do
-  (right mconcat1 . sequence)
-    <$> mapM (readJournalFile mformat mrulesfile assrt) prefixedfiles
-  where mconcat1 :: Monoid t => [t] -> t
-        mconcat1 [] = mempty
-        mconcat1 x = foldr1 mappend x
-
--- | @readJournalFile mformat mrulesfile assrt prefixedfile@
---
--- Read a Journal from this file, or from stdin if the file path is -,
--- or return an error message. The file path can have a READER: prefix.
---
--- The reader (data format) is chosen based on (in priority order):
--- the @mformat@ argument;
--- the file path's READER: prefix, if any;
--- a recognised file name extension (in readJournal);
--- if none of these identify a known reader, all built-in readers are tried in turn.
---
--- A CSV conversion rules file (@mrulesfiles@) can be specified to help convert CSV data.
---
--- Optionally, any balance assertions in the journal can be checked (@assrt@).
---
-readJournalFile :: Maybe StorageFormat -> Maybe FilePath -> Bool -> PrefixedFilePath -> IO (Either String Journal)
-readJournalFile mformat mrulesfile assrt prefixedfile = do
-  let
-    (mprefixformat, f) = splitReaderPrefix prefixedfile
-    mfmt = mformat <|> mprefixformat
-  requireJournalFileExists f
-  readFileOrStdinAnyLineEnding f >>= readJournal mfmt mrulesfile assrt (Just f)
-
 -- | If a filepath is prefixed by one of the reader names and a colon,
 -- split that off. Eg "csv:-" -> (Just "csv", "-").
 splitReaderPrefix :: PrefixedFilePath -> (Maybe String, FilePath)
@@ -195,7 +147,7 @@ newJournalContent = do
 
 -- | Read a Journal from the given text trying all readers in turn, or throw an error.
 readJournal' :: Text -> IO Journal
-readJournal' t = readJournal Nothing Nothing True Nothing t >>= either error' return
+readJournal' t = readJournal def Nothing t >>= either error' return
 
 tests_readJournal' = [
   "readJournal' parses sample journal" ~: do
@@ -203,28 +155,6 @@ tests_readJournal' = [
      assertBool "" True
  ]
 
--- | @readJournal mformat mrulesfile assrt mfile txt@
---
--- Read a Journal from some text, or return an error message.
---
--- The reader (data format) is chosen based on (in priority order):
--- the @mformat@ argument;
--- a recognised file name extension in @mfile@ (if provided).
--- If none of these identify a known reader, all built-in readers are tried in turn
--- (returning the first one's error message if none of them succeed).
---
--- A CSV conversion rules file (@mrulesfiles@) can be specified to help convert CSV data.
---
--- Optionally, any balance assertions in the journal can be checked (@assrt@).
---
-readJournal :: Maybe StorageFormat -> Maybe FilePath -> Bool -> Maybe FilePath -> Text -> IO (Either String Journal)
-readJournal mformat mrulesfile assrt mfile txt =
-  let
-    stablereaders = filter (not.rExperimental) readers
-    rs = maybe stablereaders (:[]) $ findReader mformat mfile
-  in
-    tryReaders rs mrulesfile assrt mfile txt
-
 -- | @findReader mformat mpath@
 --
 -- Find the reader named by @mformat@, if provided.
@@ -241,43 +171,41 @@ findReader Nothing (Just path) =
     (prefix,path') = splitReaderPrefix path
     ext            = drop 1 $ takeExtension path'
 
--- | @tryReaders readers mrulesfile assrt path t@
+-- | Read a Journal from each specified file path and combine them into one.
+-- Or, return the first error message.
 --
--- Try to parse the given text to a Journal using each reader in turn,
--- returning the first success, or if all of them fail, the first error message.
-tryReaders :: [Reader] -> Maybe FilePath -> Bool -> Maybe FilePath -> Text -> IO (Either String Journal)
-tryReaders readers mrulesfile assrt path t = firstSuccessOrFirstError [] readers
-  where
-    firstSuccessOrFirstError :: [String] -> [Reader] -> IO (Either String Journal)
-    firstSuccessOrFirstError [] []        = return $ Left "no readers found"
-    firstSuccessOrFirstError errs (r:rs) = do
-      dbg1IO "trying reader" (rFormat r)
-      result <- (runExceptT . (rParser r) mrulesfile assrt path') t
-      dbg1IO "reader result" $ either id show result
-      case result of Right j -> return $ Right j                        -- success!
-                     Left e  -> firstSuccessOrFirstError (errs++[e]) rs -- keep trying
-    firstSuccessOrFirstError (e:_) []    = return $ Left e              -- none left, return first error
-    path' = fromMaybe "(string)" path
-
-
---- New versions of readJournal* with easier arguments, and support for --new.
-
-readJournalFilesWithOpts :: InputOpts -> [FilePath] -> IO (Either String Journal)
-readJournalFilesWithOpts iopts =
-  (right mconcat1 . sequence <$>) . mapM (readJournalFileWithOpts iopts)
+-- Combining Journals means concatenating them, basically.
+-- The parse state resets at the start of each file, which means that
+-- directives & aliases do not affect subsequent sibling or parent files.
+-- They do affect included child files though. 
+-- Also the final parse state saved in the Journal does span all files.
+readJournalFiles :: InputOpts -> [FilePath] -> IO (Either String Journal)
+readJournalFiles iopts =
+  (right mconcat1 . sequence <$>) . mapM (readJournalFile iopts)
   where
     mconcat1 :: Monoid t => [t] -> t
     mconcat1 [] = mempty
     mconcat1 x  = foldr1 mappend x
 
-readJournalFileWithOpts :: InputOpts -> PrefixedFilePath -> IO (Either String Journal)
-readJournalFileWithOpts iopts prefixedfile = do
+-- | Read a Journal from this file, or from stdin if the file path is -,
+-- or return an error message. The file path can have a READER: prefix.
+--
+-- The reader (data format) to use is determined from (in priority order):
+-- the @mformat_@ specified in the input options, if any;
+-- the file path's READER: prefix, if any;
+-- a recognised file name extension.
+-- if none of these identify a known reader, all built-in readers are tried in turn.
+--
+-- The input options can also configure balance assertion checking, automated posting
+-- generation, a rules file for converting CSV data, etc.
+readJournalFile :: InputOpts -> PrefixedFilePath -> IO (Either String Journal)
+readJournalFile iopts prefixedfile = do
   let 
     (mfmt, f) = splitReaderPrefix prefixedfile
     iopts' = iopts{mformat_=firstJust [mfmt, mformat_ iopts]}
   requireJournalFileExists f
-  t <- readFileOrStdinAnyLineEnding f
-  ej <- readJournalWithOpts iopts' (Just f) t
+  t <- readFileOrStdinPortably f
+  ej <- readJournal iopts' (Just f) t
   case ej of
     Left e  -> return $ Left e
     Right j | new_ iopts -> do
@@ -324,7 +252,7 @@ latestDatesFileFor f = dir </> ".latest" <.> fname
     (dir, fname) = splitFileName f
 
 readFileStrictly :: FilePath -> IO Text
-readFileStrictly f = readFile' f >>= \t -> C.evaluate (T.length t) >> return t
+readFileStrictly f = readFilePortably f >>= \t -> C.evaluate (T.length t) >> return t
 
 -- | Given zero or more latest dates (all the same, representing the
 -- latest previously seen transaction date, and how many transactions
@@ -342,21 +270,40 @@ journalFilterSinceLatestDates ds@(d:_) j = (j', ds')
     j'                    = j{jtxns=newsamedatets++laterts}
     ds'                   = latestDates $ map tdate $ samedatets++laterts
 
-readJournalWithOpts :: InputOpts -> Maybe FilePath -> Text -> IO (Either String Journal)
-readJournalWithOpts iopts mfile txt =
-  tryReadersWithOpts iopts mfile specifiedorallreaders txt
+-- | @readJournal iopts mfile txt@
+--
+-- Read a Journal from some text, or return an error message.
+--
+-- The reader (data format) is chosen based on a recognised file name extension in @mfile@ (if provided).
+-- If it does not identify a known reader, all built-in readers are tried in turn
+-- (returning the first one's error message if none of them succeed).
+--
+-- Input ioptions (@iopts@) specify CSV conversion rules file to help convert CSV data,
+-- enable or disable balance assertion checking and automated posting generation.
+--
+readJournal :: InputOpts -> Maybe FilePath -> Text -> IO (Either String Journal)
+readJournal iopts mfile txt =
+  tryReaders iopts mfile specifiedorallreaders txt
   where
     specifiedorallreaders = maybe stablereaders (:[]) $ findReader (mformat_ iopts) mfile
     stablereaders = filter (not.rExperimental) readers
 
-tryReadersWithOpts :: InputOpts -> Maybe FilePath -> [Reader] -> Text -> IO (Either String Journal)
-tryReadersWithOpts iopts mpath readers txt = firstSuccessOrFirstError [] readers
+-- | @tryReaders iopts readers path t@
+--
+-- Try to parse the given text to a Journal using each reader in turn,
+-- returning the first success, or if all of them fail, the first error message.
+--    
+-- Input ioptions (@iopts@) specify CSV conversion rules file to help convert CSV data,
+-- enable or disable balance assertion checking and automated posting generation.
+--
+tryReaders :: InputOpts -> Maybe FilePath -> [Reader] -> Text -> IO (Either String Journal)
+tryReaders iopts mpath readers txt = firstSuccessOrFirstError [] readers
   where
     firstSuccessOrFirstError :: [String] -> [Reader] -> IO (Either String Journal)
     firstSuccessOrFirstError [] []        = return $ Left "no readers found"
     firstSuccessOrFirstError errs (r:rs) = do
       dbg1IO "trying reader" (rFormat r)
-      result <- (runExceptT . (rParser r) (mrules_file_ iopts) (not $ ignore_assertions_ iopts) path) txt
+      result <- (runExceptT . (rParser r) iopts path) txt
       dbg1IO "reader result" $ either id show result
       case result of Right j -> return $ Right j                        -- success!
                      Left e  -> firstSuccessOrFirstError (errs++[e]) rs -- keep trying
@@ -408,7 +355,7 @@ tests_Hledger_Read = TestList $
    "journal" ~: do
     r <- runExceptT $ parseWithState mempty JournalReader.journalp ""
     assertBool "journalp should parse an empty file" (isRight $ r)
-    jE <- readJournal Nothing Nothing True Nothing "" -- don't know how to get it from journal
+    jE <- readJournal def Nothing "" -- don't know how to get it from journal
     either error' (assertBool "journalp parsing an empty file should give an empty journal" . null . jtxns) jE
 
   ]
