@@ -11,6 +11,7 @@ A reader for CSV data, using an extra rules file to help interpret the data.
 {-# LANGUAGE ViewPatterns #-}
 {-# LANGUAGE TypeSynonymInstances #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE PackageImports #-}
 
 module Hledger.Read.CsvReader (
   -- * Reader
@@ -28,16 +29,18 @@ module Hledger.Read.CsvReader (
 )
 where
 import Prelude ()
-import Prelude.Compat hiding (getContents)
+import "base-compat-batteries" Prelude.Compat hiding (getContents)
 import Control.Exception hiding (try)
 import Control.Monad
 import Control.Monad.Except
 import Control.Monad.State.Strict (StateT, get, modify', evalStateT)
 -- import Test.HUnit
 import Data.Char (toLower, isDigit, isSpace)
-import Data.List.Compat
+import "base-compat-batteries" Data.List.Compat
+import Data.List.NonEmpty (fromList)
 import Data.Maybe
 import Data.Ord
+import qualified Data.Set as S
 import Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
@@ -53,14 +56,15 @@ import System.Directory (doesFileExist)
 import System.FilePath
 import Test.HUnit hiding (State)
 import Text.CSV (parseCSV, CSV)
-import Text.Megaparsec.Compat hiding (parse)
+import Text.Megaparsec hiding (parse)
+import Text.Megaparsec.Char
 import qualified Text.Parsec as Parsec
 import Text.Printf (printf)
 
 import Hledger.Data
 import Hledger.Utils.UTF8IOCompat (getContents)
 import Hledger.Utils
-import Hledger.Read.Common (amountp, statusp, genericSourcePos)
+import Hledger.Read.Common (Reader(..),InputOpts(..),amountp, statusp, genericSourcePos)
 
 
 reader :: Reader
@@ -73,8 +77,9 @@ reader = Reader
 
 -- | Parse and post-process a "Journal" from CSV data, or give an error.
 -- XXX currently ignores the string and reads from the file path
-parse :: Maybe FilePath -> Bool -> FilePath -> Text -> ExceptT String IO Journal
-parse rulesfile _ f t = do
+parse :: InputOpts -> FilePath -> Text -> ExceptT String IO Journal
+parse iopts f t = do
+  let rulesfile = mrules_file_ iopts
   r <- liftIO $ readJournalFromCsv rulesfile f t
   case r of Left e -> throwError e
             Right j -> return $ journalNumberAndTieTransactions j
@@ -104,7 +109,7 @@ readJournalFromCsv mrulesfile csvfile csvdata =
     if rulesfileexists
     then do
       dbg1IO "using conversion rules file" rulesfile
-      liftIO $ (readFile' rulesfile >>= expandIncludes (takeDirectory rulesfile))
+      liftIO $ (readFilePortably rulesfile >>= expandIncludes (takeDirectory rulesfile))
     else return $ defaultRulesText rulesfile
   rules <- liftIO (runExceptT $ parseAndValidateCsvRules rulesfile rulestext) >>= either throwerr return 
   dbg2IO "rules" rules
@@ -134,7 +139,7 @@ readJournalFromCsv mrulesfile csvfile csvdata =
                    (\pos r -> 
                       let
                         SourcePos name line col = pos
-                        line' = (mpMkPos . (+1) . mpUnPos) line
+                        line' = (mkPos . (+1) . unPos) line
                         pos' = SourcePos name line' col
                       in
                         (pos, transactionFromCsvRecord pos' rules r)
@@ -169,13 +174,13 @@ parseCsv path csvdata =
     "-" -> liftM (parseCSV "(stdin)") getContents
     _   -> return $ parseCSV path csvdata
 
--- | Return the cleaned up and validated CSV data, or an error.
+-- | Return the cleaned up and validated CSV data (can be empty), or an error.
 validateCsv :: Int -> Either Parsec.ParseError CSV -> Either String [CsvRecord]
 validateCsv _ (Left e) = Left $ show e
 validateCsv numhdrlines (Right rs) = validate $ drop numhdrlines $ filternulls rs
   where
     filternulls = filter (/=[""])
-    validate [] = Left "no CSV records found"
+    validate [] = Right []
     validate rs@(first:_)
       | isJust lessthan2 = let r = fromJust lessthan2 in Left $ printf "CSV record %s has less than two fields" (show r)
       | isJust different = let r = fromJust different in Left $ printf "the first CSV record %s has %d fields but %s has %d" (show first) length1 (show r) (length r)
@@ -365,7 +370,7 @@ instance ShowErrorComponent String where
 -- and runs some extra validation checks.
 parseRulesFile :: FilePath -> ExceptT String IO CsvRules
 parseRulesFile f = 
-  liftIO (readFile' f >>= expandIncludes (takeDirectory f)) >>= parseAndValidateCsvRules f
+  liftIO (readFilePortably f >>= expandIncludes (takeDirectory f)) >>= parseAndValidateCsvRules f
 
 -- | Inline all files referenced by include directives in this hledger CSV rules text, recursively.
 -- Included file paths may be relative to the directory of the provided file path.
@@ -390,11 +395,15 @@ parseAndValidateCsvRules rulesfile s = do
     Right r -> do
                r_ <- liftIO $ runExceptT $ validateRules r
                ExceptT $ case r_ of
-                 Left  s -> return $ Left $ parseErrorPretty $ mpMkParseError rulesfile s
+                 Left  s -> return $ Left $ parseErrorPretty $ makeParseError rulesfile s
                  Right r -> return $ Right r
 
+  where
+    makeParseError :: FilePath -> String -> ParseError Char String
+    makeParseError f s = FancyError (fromList [initialPos f]) (S.singleton $ ErrorFail s)
+
 -- | Parse this text as CSV conversion rules. The file path is for error messages.
-parseCsvRules :: FilePath -> T.Text -> Either (ParseError Char MPErr) CsvRules
+parseCsvRules :: FilePath -> T.Text -> Either (ParseError Char CustomErr) CsvRules
 -- parseCsvRules rulesfile s = runParser csvrulesfile nullrules{baseAccount=takeBaseName rulesfile} rulesfile s
 parseCsvRules rulesfile s =
   runParser (evalStateT rulesp rules) rulesfile s
@@ -435,10 +444,10 @@ blankorcommentlinep :: CsvRulesParser ()
 blankorcommentlinep = lift (pdbg 3 "trying blankorcommentlinep") >> choiceInState [blanklinep, commentlinep]
 
 blanklinep :: CsvRulesParser ()
-blanklinep = lift (many spacenonewline) >> newline >> return () <?> "blank line"
+blanklinep = lift (skipMany spacenonewline) >> newline >> return () <?> "blank line"
 
 commentlinep :: CsvRulesParser ()
-commentlinep = lift (many spacenonewline) >> commentcharp >> lift restofline >> return () <?> "comment line"
+commentlinep = lift (skipMany spacenonewline) >> commentcharp >> lift restofline >> return () <?> "comment line"
 
 commentcharp :: CsvRulesParser Char
 commentcharp = oneOf (";#*" :: [Char])
@@ -446,9 +455,9 @@ commentcharp = oneOf (";#*" :: [Char])
 directivep :: CsvRulesParser (DirectiveName, String)
 directivep = (do
   lift $ pdbg 3 "trying directive"
-  d <- fmap T.unpack $ choiceInState $ map (lift . mptext . T.pack) directives
+  d <- fmap T.unpack $ choiceInState $ map (lift . string . T.pack) directives
   v <- (((char ':' >> lift (many spacenonewline)) <|> lift (some spacenonewline)) >> directivevalp)
-       <|> (optional (char ':') >> lift (many spacenonewline) >> lift eolof >> return "")
+       <|> (optional (char ':') >> lift (skipMany spacenonewline) >> lift eolof >> return "")
   return (d, v)
   ) <?> "directive"
 
@@ -471,8 +480,8 @@ fieldnamelistp = (do
   lift $ pdbg 3 "trying fieldnamelist"
   string "fields"
   optional $ char ':'
-  lift (some spacenonewline)
-  let separator = lift (many spacenonewline) >> char ',' >> lift (many spacenonewline)
+  lift (skipSome spacenonewline)
+  let separator = lift (skipMany spacenonewline) >> char ',' >> lift (skipMany spacenonewline)
   f <- fromMaybe "" <$> optional fieldnamep
   fs <- some $ (separator >> fromMaybe "" <$> optional fieldnamep)
   lift restofline
@@ -504,7 +513,7 @@ fieldassignmentp = do
 journalfieldnamep :: CsvRulesParser String
 journalfieldnamep = do
   lift (pdbg 2 "trying journalfieldnamep")
-  T.unpack <$> choiceInState (map (lift . mptext . T.pack) journalfieldnames)
+  T.unpack <$> choiceInState (map (lift . string . T.pack) journalfieldnames)
 
 -- Transaction fields and pseudo fields for CSV conversion. 
 -- Names must precede any other name they contain, for the parser 
@@ -529,11 +538,11 @@ assignmentseparatorp :: CsvRulesParser ()
 assignmentseparatorp = do
   lift $ pdbg 3 "trying assignmentseparatorp"
   choice [
-    -- try (lift (many spacenonewline) >> oneOf ":="),
-    try (lift (many spacenonewline) >> char ':'),
+    -- try (lift (skipMany spacenonewline) >> oneOf ":="),
+    try (lift (skipMany spacenonewline) >> char ':'),
     spaceChar
     ]
-  _ <- lift (many spacenonewline)
+  _ <- lift (skipMany spacenonewline)
   return ()
 
 fieldvalp :: CsvRulesParser String
@@ -544,9 +553,9 @@ fieldvalp = do
 conditionalblockp :: CsvRulesParser ConditionalBlock
 conditionalblockp = do
   lift $ pdbg 3 "trying conditionalblockp"
-  string "if" >> lift (many spacenonewline) >> optional newline
+  string "if" >> lift (skipMany spacenonewline) >> optional newline
   ms <- some recordmatcherp
-  as <- many (lift (some spacenonewline) >> fieldassignmentp)
+  as <- many (lift (skipSome spacenonewline) >> fieldassignmentp)
   when (null as) $
     fail "start of conditional block found, but no assignment rules afterward\n(assignment rules in a conditional block should be indented)\n"
   return (ms, as)
@@ -556,7 +565,7 @@ recordmatcherp :: CsvRulesParser [String]
 recordmatcherp = do
   lift $ pdbg 2 "trying recordmatcherp"
   -- pos <- currentPos
-  _  <- optional (matchoperatorp >> lift (many spacenonewline) >> optional newline)
+  _  <- optional (matchoperatorp >> lift (skipMany spacenonewline) >> optional newline)
   ps <- patternsp
   when (null ps) $
     fail "start of record matcher found, but no patterns afterward\n(patterns should not be indented)\n"
@@ -564,7 +573,7 @@ recordmatcherp = do
   <?> "record matcher"
 
 matchoperatorp :: CsvRulesParser String
-matchoperatorp = fmap T.unpack $ choiceInState $ map mptext
+matchoperatorp = fmap T.unpack $ choiceInState $ map string
   ["~"
   -- ,"!~"
   -- ,"="
@@ -589,10 +598,10 @@ regexp = do
 --   pdbg 2 "trying fieldmatcher"
 --   f <- fromMaybe "all" `fmap` (optional $ do
 --          f' <- fieldname
---          lift (many spacenonewline)
+--          lift (skipMany spacenonewline)
 --          return f')
 --   char '~'
---   lift (many spacenonewline)
+--   lift (skipMany spacenonewline)
 --   ps <- patterns
 --   let r = "(" ++ intercalate "|" ps ++ ")"
 --   return (f,r)
@@ -716,19 +725,41 @@ getAmountStr rules record =
   case (render mamount, render mamountin, render mamountout) of
     (Just "", Nothing, Nothing) -> Nothing
     (Just a,  Nothing, Nothing) -> Just a
-    (Nothing, Just "", Just "") -> error' $ "neither amount-in or amount-out has a value\n"++showRecord record
+    (Nothing, Just "", Just "") -> error' $    "neither amount-in or amount-out has a value\n"
+                                            ++ "    record: " ++ showRecord record
     (Nothing, Just i,  Just "") -> Just i
     (Nothing, Just "", Just o)  -> Just $ negateStr o
-    (Nothing, Just _,  Just _)  -> error' $ "both amount-in and amount-out have a value\n"++showRecord record
-    _                           -> error' $ "found values for amount and for amount-in/amount-out - please use either amount or amount-in/amount-out\n"++showRecord record
+    (Nothing, Just i,  Just o)  -> error' $    "both amount-in and amount-out have a value\n"
+                                            ++ "    amount-in: "  ++ i ++ "\n"
+                                            ++ "    amount-out: " ++ o ++ "\n"
+                                            ++ "    record: "     ++ showRecord record
+    _                           -> error' $    "found values for amount and for amount-in/amount-out\n"
+                                            ++ "please use either amount or amount-in/amount-out\n"
+                                            ++ "    record: " ++ showRecord record
 
 type CsvAmountString = String
 
 -- | Canonicalise the sign in a CSV amount string.
--- Such strings can be parenthesized, which is equivalent to having a minus sign.
--- Also they can end up with a double minus sign, which cancels out.
+-- Such strings can have a minus sign, negating parentheses, 
+-- or any two of these (which cancels out).
+--
+-- >>> simplifySign "1"
+-- "1"
+-- >>> simplifySign "-1"
+-- "-1"
+-- >>> simplifySign "(1)"
+-- "-1"
+-- >>> simplifySign "--1"
+-- "1"
+-- >>> simplifySign "-(1)"
+-- "1"
+-- >>> simplifySign "(-1)"
+-- "1"
+-- >>> simplifySign "((1))"
+-- "1"
 simplifySign :: CsvAmountString -> CsvAmountString
 simplifySign ('(':s) | lastMay s == Just ')' = simplifySign $ negateStr $ init s
+simplifySign ('-':'(':s) | lastMay s == Just ')' = simplifySign $ init s
 simplifySign ('-':'-':s) = s
 simplifySign s = s
 
