@@ -29,7 +29,7 @@ import cycles.
 
 --- * module
 
-{-# LANGUAGE CPP, RecordWildCards, NamedFieldPuns, NoMonoLocalBinds, ScopedTypeVariables, FlexibleContexts, TupleSections, OverloadedStrings #-}
+{-# LANGUAGE CPP, RecordWildCards, NamedFieldPuns, NoMonoLocalBinds, ScopedTypeVariables, FlexibleContexts, TupleSections, OverloadedStrings, PackageImports #-}
 
 module Hledger.Read.JournalReader (
 --- * exports
@@ -42,8 +42,6 @@ module Hledger.Read.JournalReader (
   parseAndFinaliseJournal,
   runJournalParser,
   rjp,
-  runErroringJournalParser,
-  rejp,
 
   -- * Parsers used elsewhere
   getParentAccount,
@@ -63,8 +61,7 @@ module Hledger.Read.JournalReader (
   -- numberp,
   statusp,
   emptyorcommentlinep,
-  followingcommentp,
-  accountaliasp
+  followingcommentp
 
   -- * Tests
   ,tests_Hledger_Read_JournalReader
@@ -73,13 +70,13 @@ module Hledger.Read.JournalReader (
 where
 --- * imports
 import Prelude ()
-import Prelude.Compat hiding (readFile)
+import "base-compat-batteries" Prelude.Compat hiding (readFile)
 import qualified Control.Exception as C
 import Control.Monad
-import Control.Monad.Except (ExceptT(..), runExceptT, throwError)
+import Control.Monad.Except (ExceptT(..))
 import Control.Monad.State.Strict
+import Data.Bifunctor (first)
 import qualified Data.Map.Strict as M
-import Data.Monoid
 import Data.Text (Text)
 import Data.String
 import Data.List
@@ -92,7 +89,9 @@ import Test.HUnit
 import Test.Framework
 import Text.Megaparsec.Error
 #endif
-import Text.Megaparsec.Compat hiding (parse)
+import Text.Megaparsec hiding (parse)
+import Text.Megaparsec.Char
+import Text.Megaparsec.Custom
 import Text.Printf
 import System.FilePath
 
@@ -117,8 +116,18 @@ reader = Reader
 
 -- | Parse and post-process a "Journal" from hledger's journal file
 -- format, or give an error.
-parse :: Maybe FilePath -> Bool -> FilePath -> Text -> ExceptT String IO Journal
-parse _ = parseAndFinaliseJournal journalp
+parse :: InputOpts -> FilePath -> Text -> ExceptT String IO Journal
+parse iopts = parseAndFinaliseJournal journalp' iopts
+  where
+    journalp' = do 
+      -- reverse parsed aliases to ensure that they are applied in order given on commandline
+      mapM_ addAccountAlias (reverse $ aliasesFromOpts iopts) 
+      journalp
+
+-- | Get the account name aliases from options, if any.
+aliasesFromOpts :: InputOpts -> [AccountAlias]
+aliasesFromOpts = map (\a -> fromparse $ runParser accountaliasp ("--alias "++quoteIfNeeded a) $ T.pack a)
+                  . aliases_
 
 --- * parsers
 --- ** journal
@@ -126,10 +135,10 @@ parse _ = parseAndFinaliseJournal journalp
 -- | A journal parser. Accumulates and returns a "ParsedJournal",
 -- which should be finalised/validated before use.
 --
--- >>> rejp (journalp <* eof) "2015/1/1\n a  0\n"
+-- >>> rjp (journalp <* eof) "2015/1/1\n a  0\n"
 -- Right Journal  with 1 transactions, 1 accounts
 --
-journalp :: MonadIO m => ErroringJournalParser m ParsedJournal
+journalp :: MonadIO m => JournalParser m ParsedJournal
 journalp = do
   many addJournalItemP
   eof
@@ -137,7 +146,7 @@ journalp = do
 
 -- | A side-effecting parser; parses any kind of journal item
 -- and updates the parse state accordingly.
-addJournalItemP :: MonadIO m => ErroringJournalParser m ()
+addJournalItemP :: MonadIO m => JournalParser m ()
 addJournalItemP =
   -- all journal line types can be distinguished by the first
   -- character, can use choice without backtracking
@@ -147,8 +156,8 @@ addJournalItemP =
     , modifiertransactionp  >>= modify' . addModifierTransaction
     , periodictransactionp  >>= modify' . addPeriodicTransaction
     , marketpricedirectivep >>= modify' . addMarketPrice
-    , void emptyorcommentlinep
-    , void multilinecommentp
+    , void (lift emptyorcommentlinep)
+    , void (lift multilinecommentp)
     ] <?> "transaction or directive"
 
 --- ** directives
@@ -156,7 +165,7 @@ addJournalItemP =
 -- | Parse any journal directive and update the parse state accordingly.
 -- Cf http://hledger.org/manual.html#directives,
 -- http://ledger-cli.org/3.0/doc/ledger3.html#Command-Directives
-directivep :: MonadIO m => ErroringJournalParser m ()
+directivep :: MonadIO m => JournalParser m ()
 directivep = (do
   optional $ char '!'
   choice [
@@ -176,40 +185,44 @@ directivep = (do
    ]
   ) <?> "directive"
 
-includedirectivep :: MonadIO m => ErroringJournalParser m ()
+includedirectivep :: MonadIO m => JournalParser m ()
 includedirectivep = do
   string "include"
-  lift (some spacenonewline)
-  filename  <- lift restofline
-  parentpos <- getPosition
-  parentj   <- get
+  lift (skipSome spacenonewline)
+  filename <- T.unpack <$> takeWhileP Nothing (/= '\n') -- don't consume newline yet
+
+  -- save parent state
+  parentParserState <- getParserState
+  parentj <- get
+
   let childj = newJournalWithParseStateFrom parentj
-  (ej :: Either String ParsedJournal) <-
-    liftIO $ runExceptT $ do
-      let curdir = takeDirectory (sourceName parentpos)
-      filepath <- expandPath curdir filename `orRethrowIOError` (show parentpos ++ " locating " ++ filename)
-      txt      <- readFileAnyLineEnding filepath `orRethrowIOError` (show parentpos ++ " reading " ++ filepath)
-      (ej1::Either (ParseError Char MPErr) ParsedJournal) <-
-        runParserT
-           (evalStateT
-              (choiceInState
-                 [journalp
-                 ,timeclockfilep
-                 ,timedotfilep
-                 -- can't include a csv file yet, that reader is special
-                 ])
-              childj)
-           filepath txt
-      either
-        (throwError
-          . ((show parentpos ++ " in included file " ++ show filename ++ ":\n") ++)
-          . parseErrorPretty)
-        (return . journalAddFile (filepath, txt))
-        ej1
-  case ej of
-    Left e       -> throwError e
-    Right childj -> modify' (\parentj -> childj <> parentj)
-    -- discard child's parse info, prepend its (reversed) list data, combine other fields
+  parentpos <- getPosition
+
+  -- read child input
+  let curdir = takeDirectory (sourceName parentpos)
+  filepath <- lift $ expandPath curdir filename `orRethrowIOError` (show parentpos ++ " locating " ++ filename)
+  childInput <- lift $ readFilePortably filepath `orRethrowIOError` (show parentpos ++ " reading " ++ filepath)
+
+  -- set child state
+  setInput childInput
+  pushPosition $ initialPos filepath
+  put childj
+
+  -- parse include file
+  let parsers = [ journalp
+                , timeclockfilep
+                , timedotfilep
+                ] -- can't include a csv file yet, that reader is special
+  updatedChildj <- journalAddFile (filepath, childInput) <$>
+                   region (withSource childInput) (choiceInState parsers)
+
+  -- restore parent state, prepending the child's parse info
+  setParserState parentParserState
+  put $ updatedChildj <> parentj
+  -- discard child's parse info, prepend its (reversed) list data, combine other fields
+
+  void newline
+
 
 newJournalWithParseStateFrom :: Journal -> Journal
 newJournalWithParseStateFrom j = mempty{
@@ -224,82 +237,96 @@ newJournalWithParseStateFrom j = mempty{
 
 -- | Lift an IO action into the exception monad, rethrowing any IO
 -- error with the given message prepended.
-orRethrowIOError :: IO a -> String -> ExceptT String IO a
-orRethrowIOError io msg =
-  ExceptT $
-    (Right <$> io)
-    `C.catch` \(e::C.IOException) -> return $ Left $ printf "%s:\n%s" msg (show e)
+orRethrowIOError :: MonadIO m => IO a -> String -> TextParser m a
+orRethrowIOError io msg = do
+  eResult <- liftIO $ (Right <$> io) `C.catch` \(e::C.IOException) -> pure $ Left $ printf "%s:\n%s" msg (show e)
+  case eResult of
+    Right res -> pure res
+    Left errMsg -> fail errMsg
 
 accountdirectivep :: JournalParser m ()
 accountdirectivep = do
   string "account"
-  lift (some spacenonewline)
-  acct <- lift accountnamep
+  lift (skipSome spacenonewline)
+  acct <- modifiedaccountnamep  -- account directives can be modified by alias/apply account
+  macode' :: Maybe String <- (optional $ lift $ skipSome spacenonewline >> some digitChar)
+  let macode :: Maybe AccountCode = read <$> macode'
   newline
-  many indentedlinep
-  modify' (\j -> j{jaccounts = acct : jaccounts j})
-
+  skipMany indentedlinep
+    
+  modify' (\j -> j{jaccounts = (acct, macode) : jaccounts j})
 
 indentedlinep :: JournalParser m String
-indentedlinep = lift (some spacenonewline) >> (rstrip <$> lift restofline)
+indentedlinep = lift (skipSome spacenonewline) >> (rstrip <$> lift restofline)
 
 -- | Parse a one-line or multi-line commodity directive.
 --
--- >>> Right _ <- rejp commoditydirectivep "commodity $1.00"
--- >>> Right _ <- rejp commoditydirectivep "commodity $\n  format $1.00"
--- >>> Right _ <- rejp commoditydirectivep "commodity $\n\n" -- a commodity with no format
--- >>> Right _ <- rejp commoditydirectivep "commodity $1.00\n  format $1.00" -- both, what happens ?
-commoditydirectivep :: Monad m => ErroringJournalParser m ()
-commoditydirectivep = try commoditydirectiveonelinep <|> commoditydirectivemultilinep
+-- >>> Right _ <- rjp commoditydirectivep "commodity $1.00"
+-- >>> Right _ <- rjp commoditydirectivep "commodity $\n  format $1.00"
+-- >>> Right _ <- rjp commoditydirectivep "commodity $\n\n" -- a commodity with no format
+-- >>> Right _ <- rjp commoditydirectivep "commodity $1.00\n  format $1.00" -- both, what happens ?
+commoditydirectivep :: JournalParser m ()
+commoditydirectivep = commoditydirectiveonelinep <|> commoditydirectivemultilinep
 
 -- | Parse a one-line commodity directive.
 --
--- >>> Right _ <- rejp commoditydirectiveonelinep "commodity $1.00"
--- >>> Right _ <- rejp commoditydirectiveonelinep "commodity $1.00 ; blah\n"
-commoditydirectiveonelinep :: Monad m => JournalParser m ()
+-- >>> Right _ <- rjp commoditydirectiveonelinep "commodity $1.00"
+-- >>> Right _ <- rjp commoditydirectiveonelinep "commodity $1.00 ; blah\n"
+commoditydirectiveonelinep :: JournalParser m ()
 commoditydirectiveonelinep = do
-  string "commodity"
-  lift (some spacenonewline)
-  Amount{acommodity,astyle} <- amountp
-  lift (many spacenonewline)
-  _ <- followingcommentp <|> (lift eolof >> return "")
-  let comm = Commodity{csymbol=acommodity, cformat=Just astyle}
-  modify' (\j -> j{jcommodities=M.insert acommodity comm $ jcommodities j})
+  (pos, Amount{acommodity,astyle}) <- try $ do
+    string "commodity"
+    lift (skipSome spacenonewline)
+    pos <- getPosition
+    amount <- amountp
+    pure $ (pos, amount)
+  lift (skipMany spacenonewline)
+  _ <- lift followingcommentp
+  let comm = Commodity{csymbol=acommodity, cformat=Just $ dbg2 "style from commodity directive" astyle}
+  if asdecimalpoint astyle == Nothing
+  then parseErrorAt pos pleaseincludedecimalpoint
+  else modify' (\j -> j{jcommodities=M.insert acommodity comm $ jcommodities j})
+
+pleaseincludedecimalpoint :: String
+pleaseincludedecimalpoint = "to avoid ambiguity, please include a decimal point in commodity directives"
 
 -- | Parse a multi-line commodity directive, containing 0 or more format subdirectives.
 --
--- >>> Right _ <- rejp commoditydirectivemultilinep "commodity $ ; blah \n  format $1.00 ; blah"
-commoditydirectivemultilinep :: Monad m => ErroringJournalParser m ()
+-- >>> Right _ <- rjp commoditydirectivemultilinep "commodity $ ; blah \n  format $1.00 ; blah"
+commoditydirectivemultilinep :: JournalParser m ()
 commoditydirectivemultilinep = do
   string "commodity"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   sym <- lift commoditysymbolp
-  _ <- followingcommentp <|> (lift eolof >> return "")
+  _ <- lift followingcommentp
   mformat <- lastMay <$> many (indented $ formatdirectivep sym)
   let comm = Commodity{csymbol=sym, cformat=mformat}
   modify' (\j -> j{jcommodities=M.insert sym comm $ jcommodities j})
   where
-    indented = (lift (some spacenonewline) >>)
+    indented = (lift (skipSome spacenonewline) >>)
 
 -- | Parse a format (sub)directive, throwing a parse error if its
 -- symbol does not match the one given.
-formatdirectivep :: Monad m => CommoditySymbol -> ErroringJournalParser m AmountStyle
+formatdirectivep :: CommoditySymbol -> JournalParser m AmountStyle
 formatdirectivep expectedsym = do
   string "format"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   pos <- getPosition
   Amount{acommodity,astyle} <- amountp
-  _ <- followingcommentp <|> (lift eolof >> return "")
+  _ <- lift followingcommentp
   if acommodity==expectedsym
-    then return astyle
-    else parserErrorAt pos $
+    then 
+      if asdecimalpoint astyle == Nothing
+      then parseErrorAt pos pleaseincludedecimalpoint
+      else return $ dbg2 "style from format subdirective" astyle
+    else parseErrorAt pos $
          printf "commodity directive symbol \"%s\" and format directive symbol \"%s\" should be the same" expectedsym acommodity
 
 keywordp :: String -> JournalParser m ()
 keywordp = (() <$) . string . fromString
 
 spacesp :: JournalParser m ()
-spacesp = () <$ lift (some spacenonewline)
+spacesp = () <$ lift (skipSome spacenonewline)
 
 -- | Backtracking parser similar to string, but allows varying amount of space between words
 keywordsp :: String -> JournalParser m ()
@@ -308,7 +335,7 @@ keywordsp = try . sequence_ . intersperse spacesp . map keywordp . words
 applyaccountdirectivep :: JournalParser m ()
 applyaccountdirectivep = do
   keywordsp "apply account" <?> "apply account directive"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   parent <- lift accountnamep
   newline
   pushParentAccount parent
@@ -321,7 +348,7 @@ endapplyaccountdirectivep = do
 aliasdirectivep :: JournalParser m ()
 aliasdirectivep = do
   string "alias"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   alias <- lift accountaliasp
   addAccountAlias alias
 
@@ -333,7 +360,7 @@ basicaliasp = do
   -- pdbg 0 "basicaliasp"
   old <- rstrip <$> (some $ noneOf ("=" :: [Char]))
   char '='
-  many spacenonewline
+  skipMany spacenonewline
   new <- rstrip <$> anyChar `manyTill` eolof  -- eol in journal, eof in command lines, normally
   return $ BasicAlias (T.pack old) (T.pack new)
 
@@ -343,9 +370,9 @@ regexaliasp = do
   char '/'
   re <- some $ noneOf ("/\n\r" :: [Char]) -- paranoid: don't try to read past line end
   char '/'
-  many spacenonewline
+  skipMany spacenonewline
   char '='
-  many spacenonewline
+  skipMany spacenonewline
   repl <- anyChar `manyTill` eolof
   return $ RegexAlias re repl
 
@@ -357,7 +384,7 @@ endaliasesdirectivep = do
 tagdirectivep :: JournalParser m ()
 tagdirectivep = do
   string "tag" <?> "tag directive"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   _ <- lift $ some nonspace
   lift restofline
   return ()
@@ -371,28 +398,31 @@ endtagdirectivep = do
 defaultyeardirectivep :: JournalParser m ()
 defaultyeardirectivep = do
   char 'Y' <?> "default year"
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   y <- some digitChar
   let y' = read y
   failIfInvalidYear y
   setYear y'
 
-defaultcommoditydirectivep :: Monad m => JournalParser m ()
+defaultcommoditydirectivep :: JournalParser m ()
 defaultcommoditydirectivep = do
   char 'D' <?> "default commodity"
-  lift (some spacenonewline)
-  Amount{..} <- amountp
+  lift (skipSome spacenonewline)
+  pos <- getPosition
+  Amount{acommodity,astyle} <- amountp
   lift restofline
-  setDefaultCommodityAndStyle (acommodity, astyle)
+  if asdecimalpoint astyle == Nothing
+  then parseErrorAt pos pleaseincludedecimalpoint
+  else setDefaultCommodityAndStyle (acommodity, astyle)
 
-marketpricedirectivep :: Monad m => JournalParser m MarketPrice
+marketpricedirectivep :: JournalParser m MarketPrice
 marketpricedirectivep = do
   char 'P' <?> "market price"
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   date <- try (do {LocalTime d _ <- datetimep; return d}) <|> datep -- a time is ignored
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   symbol <- lift commoditysymbolp
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   price <- amountp
   lift restofline
   return $ MarketPrice date symbol price
@@ -400,57 +430,93 @@ marketpricedirectivep = do
 ignoredpricecommoditydirectivep :: JournalParser m ()
 ignoredpricecommoditydirectivep = do
   char 'N' <?> "ignored-price commodity"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   lift commoditysymbolp
   lift restofline
   return ()
 
-commodityconversiondirectivep :: Monad m => JournalParser m ()
+commodityconversiondirectivep :: JournalParser m ()
 commodityconversiondirectivep = do
   char 'C' <?> "commodity conversion"
-  lift (some spacenonewline)
+  lift (skipSome spacenonewline)
   amountp
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   char '='
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   amountp
   lift restofline
   return ()
 
 --- ** transactions
 
-modifiertransactionp :: MonadIO m => ErroringJournalParser m ModifierTransaction
+modifiertransactionp :: JournalParser m ModifierTransaction
 modifiertransactionp = do
   char '=' <?> "modifier transaction"
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   valueexpr <- T.pack <$> lift restofline
   postings <- postingsp Nothing
   return $ ModifierTransaction valueexpr postings
 
-periodictransactionp :: MonadIO m => ErroringJournalParser m PeriodicTransaction
+-- | Parse a periodic transaction
+periodictransactionp :: MonadIO m => JournalParser m PeriodicTransaction
 periodictransactionp = do
+
+  -- first line
   char '~' <?> "periodic transaction"
-  lift (many spacenonewline)
-  periodexpr <- T.pack <$> lift restofline
-  postings <- postingsp Nothing
-  return $ PeriodicTransaction periodexpr postings
+  lift $ skipMany spacenonewline
+  -- a period expression
+  pos <- getPosition
+  d <- liftIO getCurrentDay
+  (periodtxt, (interval, span)) <- lift $ first T.strip <$> match (periodexprp d)
+  -- In periodic transactions, the period expression has an additional constraint:
+  case checkPeriodicTransactionStartDate interval span periodtxt of
+    Just e -> parseErrorAt pos e
+    Nothing -> pure ()
+  -- The line can end here, or it can continue with one or more spaces
+  -- and then zero or more of the following fields. A bit awkward.
+  (status, code, description, (comment, tags)) <-
+    (lift eolof >> return (Unmarked, "", "", ("", [])))
+    <|>
+    (do
+      lift $ skipSome spacenonewline 
+      s         <- lift statusp
+      c         <- lift codep
+      desc      <- lift $ T.strip <$> descriptionp
+      (cmt, ts) <- lift transactioncommentp
+      return (s,c,desc,(cmt,ts))
+    )
+
+  -- next lines
+  postings <- postingsp (Just $ first3 $ toGregorian d)
+
+  return $ nullperiodictransaction{
+     ptperiodexpr=periodtxt
+    ,ptinterval=interval
+    ,ptspan=span
+    ,ptstatus=status
+    ,ptcode=code
+    ,ptdescription=description
+    ,ptcomment=comment
+    ,pttags=tags
+    ,ptpostings=postings
+    }
 
 -- | Parse a (possibly unbalanced) transaction.
-transactionp :: MonadIO m => ErroringJournalParser m Transaction
+transactionp :: JournalParser m Transaction
 transactionp = do
   -- ptrace "transactionp"
-  pos <- getPosition
+  startpos <- getPosition
   date <- datep <?> "transaction"
-  edate <- optional (secondarydatep date) <?> "secondary date"
+  edate <- optional (lift $ secondarydatep date) <?> "secondary date"
   lookAhead (lift spacenonewline <|> newline) <?> "whitespace or newline"
   status <- lift statusp <?> "cleared status"
-  code <- T.pack <$> lift codep <?> "transaction code"
-  description <- T.pack . strip <$> descriptionp
-  comment <- try followingcommentp <|> (newline >> return "")
-  let tags = commentTags comment
-  postings <- postingsp (Just date)
-  pos' <-  getPosition
-  let sourcepos = journalSourcePos pos pos'
+  code <- lift codep <?> "transaction code"
+  description <- lift $ T.strip <$> descriptionp
+  (comment, tags) <- lift transactioncommentp
+  let year = first3 $ toGregorian date
+  postings <- postingsp (Just year)
+  endpos <- getPosition
+  let sourcepos = journalSourcePos startpos endpos
   return $ txnTieKnot $ Transaction 0 sourcepos date edate status code description comment tags postings ""
 
 #ifdef TESTS
@@ -550,30 +616,32 @@ test_transactionp = do
 
 -- Parse the following whitespace-beginning lines as postings, posting
 -- tags, and/or comments (inferring year, if needed, from the given date).
-postingsp :: MonadIO m => Maybe Day -> ErroringJournalParser m [Posting]
-postingsp mdate = many (try $ postingp mdate) <?> "postings"
+postingsp :: Maybe Year -> JournalParser m [Posting]
+postingsp mTransactionYear = many (postingp mTransactionYear) <?> "postings"
 
--- linebeginningwithspaces :: Monad m => JournalParser m String
+-- linebeginningwithspaces :: JournalParser m String
 -- linebeginningwithspaces = do
---   sp <- lift (some spacenonewline)
+--   sp <- lift (skipSome spacenonewline)
 --   c <- nonspace
 --   cs <- lift restofline
 --   return $ sp ++ (c:cs) ++ "\n"
 
-postingp :: MonadIO m => Maybe Day -> ErroringJournalParser m Posting
-postingp mtdate = do
+postingp :: Maybe Year -> JournalParser m Posting
+postingp mTransactionYear = do
   -- pdbg 0 "postingp"
-  lift (some spacenonewline)
-  status <- lift statusp
-  lift (many spacenonewline)
-  account <- modifiedaccountnamep
+  (status, account) <- try $ do
+    lift (skipSome spacenonewline)
+    status <- lift statusp
+    lift (skipMany spacenonewline)
+    account <- modifiedaccountnamep
+    return (status, account)
   let (ptype, account') = (accountNamePostingType account, textUnbracket account)
-  amount <- spaceandamountormissingp
+  lift (skipMany spacenonewline)
+  amount <- option missingmixedamt $ Mixed . (:[]) <$> amountp
   massertion <- partialbalanceassertionp
   _ <- fixedlotpricep
-  lift (many spacenonewline)
-  (comment,tags,mdate,mdate2) <-
-    try (followingcommentandtagsp mtdate) <|> (newline >> return ("",[],Nothing,Nothing))
+  lift (skipMany spacenonewline)
+  (comment,tags,mdate,mdate2) <- lift $ postingcommentp mTransactionYear
   return posting
    { pdate=mdate
    , pdate2=mdate2
