@@ -23,7 +23,7 @@ inc.client1   .... .... ..
 
 -}
 
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings, PackageImports #-}
 
 module Hledger.Read.TimedotReader (
   -- * Reader
@@ -35,7 +35,7 @@ module Hledger.Read.TimedotReader (
 )
 where
 import Prelude ()
-import Prelude.Compat
+import "base-compat-batteries" Prelude.Compat
 import Control.Monad
 import Control.Monad.Except (ExceptT)
 import Control.Monad.State.Strict
@@ -44,7 +44,8 @@ import Data.List (foldl')
 import Data.Maybe
 import Data.Text (Text)
 import Test.HUnit
-import Text.Megaparsec.Compat hiding (parse)
+import Text.Megaparsec hiding (parse)
+import Text.Megaparsec.Char
 
 import Hledger.Data
 import Hledger.Read.Common
@@ -65,8 +66,8 @@ reader = Reader
   }
 
 -- | Parse and post-process a "Journal" from the timedot format, or give an error.
-parse :: Maybe FilePath -> Bool -> FilePath -> Text -> ExceptT String IO Journal
-parse _ = parseAndFinaliseJournal timedotfilep
+parse :: InputOpts -> FilePath -> Text -> ExceptT String IO Journal
+parse = parseAndFinaliseJournal timedotfilep
 
 timedotfilep :: JournalParser m ParsedJournal
 timedotfilep = do many timedotfileitemp
@@ -77,7 +78,7 @@ timedotfilep = do many timedotfileitemp
       timedotfileitemp = do
         ptrace "timedotfileitemp"
         choice [
-          void emptyorcommentlinep
+          void $ lift emptyorcommentlinep
          ,timedotdayp >>= \ts -> modify' (addTransactions ts)
          ] <?> "timedot day entry, or default year or comment line or blank line"
 
@@ -95,7 +96,7 @@ timedotdayp :: JournalParser m [Transaction]
 timedotdayp = do
   ptrace " timedotdayp"
   d <- datep <* lift eolof
-  es <- catMaybes <$> many (const Nothing <$> try emptyorcommentlinep <|>
+  es <- catMaybes <$> many (const Nothing <$> try (lift emptyorcommentlinep) <|>
                             Just <$> (notFollowedBy datep >> timedotentryp))
   return $ map (\t -> t{tdate=d}) es -- <$> many timedotentryp
 
@@ -107,13 +108,13 @@ timedotentryp :: JournalParser m Transaction
 timedotentryp = do
   ptrace "  timedotentryp"
   pos <- genericSourcePos <$> getPosition
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   a <- modifiedaccountnamep
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   hours <-
-    try (followingcommentp >> return 0)
+    try (lift followingcommentp >> return 0)
     <|> (timedotdurationp <*
-         (try followingcommentp <|> (newline >> return "")))
+         (try (lift followingcommentp) <|> (newline >> return "")))
   let t = nulltransaction{
         tsourcepos = pos,
         tstatus    = Cleared,
@@ -143,7 +144,7 @@ timedotnumericp :: JournalParser m Quantity
 timedotnumericp = do
   (q, _, _, _) <- lift $ numberp Nothing
   msymbol <- optional $ choice $ map (string . fst) timeUnits
-  lift (many spacenonewline)
+  lift (skipMany spacenonewline)
   let q' = 
         case msymbol of
           Nothing  -> q
