@@ -9,7 +9,11 @@ module Hledger.Reports.MultiBalanceReports (
   MultiBalanceReport(..),
   MultiBalanceReportRow,
   multiBalanceReport,
-  singleBalanceReport,
+  balanceReportFromMultiBalanceReport,
+  mbrNegate,
+  mbrNormaliseSign,
+  multiBalanceReportSpan,
+  tableAsText,
 
   -- -- * Tests
   tests_Hledger_Reports_MultiBalanceReport
@@ -22,6 +26,8 @@ import Data.Ord
 import Data.Time.Calendar
 import Safe
 import Test.HUnit
+import Text.Tabular as T
+import Text.Tabular.AsciiWide
 
 import Hledger.Data
 import Hledger.Query
@@ -72,25 +78,15 @@ instance Show MultiBalanceReport where
 -- type alias just to remind us which AccountNames might be depth-clipped, below.
 type ClippedAccountName = AccountName
 
--- | Generates a single column BalanceReport like balanceReport, but uses
--- multiBalanceReport, so supports --historical. 
--- TODO Does not support boring parent eliding or --flat yet.
-singleBalanceReport :: ReportOpts -> Query -> Journal -> BalanceReport
-singleBalanceReport opts q j = (rows', total)
-  where
-    MultiBalanceReport (_, rows, (totals, _, _)) = multiBalanceReport opts q j
-    rows' = [(a
-             ,if flat_ opts then a else a'   -- BalanceReport expects full account name here with --flat
-             ,if tree_ opts then d-1 else 0  -- BalanceReport uses 0-based account depths
-             , headDef nullmixedamt amts     -- 0 columns is illegal, should not happen, return zeroes if it does
-             ) | (a,a',d, amts, _, _) <- rows]
-    total = headDef nullmixedamt totals
-
 -- | Generate a multicolumn balance report for the matched accounts,
 -- showing the change of balance, accumulated balance, or historical balance
--- in each of the specified periods.
+-- in each of the specified periods. Does not support tree-mode boring parent eliding.
+-- If the normalbalance_ option is set, it adjusts the sorting and sign of amounts 
+-- (see ReportOpts and CompoundBalanceCommand).
 multiBalanceReport :: ReportOpts -> Query -> Journal -> MultiBalanceReport
-multiBalanceReport opts q j = MultiBalanceReport (displayspans, sorteditems, totalsrow)
+multiBalanceReport opts q j =
+  (if invert_ opts then mbrNegate else id) $ 
+  MultiBalanceReport (displayspans, sorteditems, totalsrow)
     where
       symq       = dbg1 "symq"   $ filterQuery queryIsSym $ dbg1 "requested q" q
       depthq     = dbg1 "depthq" $ filterQuery queryIsDepth q
@@ -182,17 +178,18 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, sorteditems, tot
 
       sorteditems :: [MultiBalanceReportRow] =
         dbg1 "sorteditems" $
-        maybesort items
+        sortitems items
         where
-          maybesort
-            | not $ sort_amount_ opts         = id
-            | accountlistmode_ opts == ALTree = sortTreeMultiBalanceReportRowsByAmount
-            | otherwise                       = sortFlatMultiBalanceReportRowsByAmount
+          sortitems
+            | sort_amount_ opts && accountlistmode_ opts == ALTree       = sortTreeMultiBalanceReportRowsByAmount
+            | sort_amount_ opts                                          = sortFlatMultiBalanceReportRowsByAmount
+            | not (sort_amount_ opts) && accountlistmode_ opts == ALTree = sortTreeMultiBalanceReportRowsByAccountCodeAndName
+            | otherwise                                                  = sortFlatMultiBalanceReportRowsByAccountCodeAndName
             where
               -- Sort the report rows, representing a flat account list, by row total. 
               sortFlatMultiBalanceReportRowsByAmount = sortBy (maybeflip $ comparing fifth6)
                 where
-                  maybeflip = if normalbalance_ opts == Just NormalNegative then id else flip
+                  maybeflip = if normalbalance_ opts == Just NormallyNegative then id else flip
 
               -- Sort the report rows, representing a tree of accounts, by row total at each level.
               -- To do this we recreate an Account tree with the row totals as balances, 
@@ -209,13 +206,34 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, sorteditems, tot
                     where
                       -- this error should not happen, but it's ugly TODO 
                       setibalance a = a{aibalance=fromMaybe (error "sortTreeMultiBalanceReportRowsByAmount 1") $ lookup (aname a) atotals}
-                  sortedaccounttree = sortAccountTreeByAmount (fromMaybe NormalPositive $ normalbalance_ opts) accounttreewithbals
+                  sortedaccounttree = sortAccountTreeByAmount (fromMaybe NormallyPositive $ normalbalance_ opts) accounttreewithbals
                   sortedaccounts = drop 1 $ flattenAccounts sortedaccounttree
-                  sortedrows = [ 
-                    -- this error should not happen, but it's ugly TODO 
-                    fromMaybe (error "sortTreeMultiBalanceReportRowsByAmount 2") $ lookup (aname a) anamesandrows
-                    | a <- sortedaccounts 
-                    ]
+                  -- dropped the root account, also ignore any parent accounts not in rows
+                  sortedrows = concatMap (\a -> maybe [] (:[]) $ lookup (aname a) anamesandrows) sortedaccounts 
+
+              -- Sort the report rows by account code if any, with the empty account code coming last, then account name. 
+              -- TODO keep children below their parent. Have to convert to tree ? 
+              sortFlatMultiBalanceReportRowsByAccountCodeAndName = sortBy (comparing acodeandname)
+                where
+                  acodeandname r = (acode', aname)
+                    where
+                      aname = first6 r
+                      macode = fromMaybe Nothing $ lookup aname $ jaccounts j
+                      acode' = fromMaybe maxBound macode 
+
+              -- Sort the report rows, representing a tree of accounts, by account code and then account name at each level.
+              -- Convert a tree of account names, look up the account codes, sort and flatten the tree, reorder the rows.
+              sortTreeMultiBalanceReportRowsByAccountCodeAndName rows = sortedrows
+                where
+                  anamesandrows = [(first6 r, r) | r <- rows]
+                  anames = map fst anamesandrows
+                  nametree = treeFromPaths $ map expandAccountName anames
+                  accounttree = nameTreeToAccount "root" nametree
+                  accounttreewithcodes = mapAccounts (accountSetCodeFrom j) accounttree
+                  sortedaccounttree = sortAccountTreeByAccountCodeAndName accounttreewithcodes
+                  sortedaccounts = drop 1 $ flattenAccounts sortedaccounttree
+                  -- dropped the root account, also ignore any parent accounts not in rows
+                  sortedrows = concatMap (\a -> maybe [] (:[]) $ lookup (aname a) anamesandrows) sortedaccounts 
 
       totals :: [MixedAmount] =
           -- dbg1 "totals" $
@@ -233,6 +251,39 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, sorteditems, tot
       dbg1 s = let p = "multiBalanceReport" in Hledger.Utils.dbg1 (p++" "++s)  -- add prefix in this function's debug output
       -- dbg1 = const id  -- exclude this function from debug output
 
+-- | Given a MultiBalanceReport and its normal balance sign,
+-- if it is known to be normally negative, convert it to normally positive.
+mbrNormaliseSign :: NormalSign -> MultiBalanceReport -> MultiBalanceReport
+mbrNormaliseSign NormallyNegative = mbrNegate
+mbrNormaliseSign _ = id
+
+-- | Flip the sign of all amounts in a MultiBalanceReport.
+mbrNegate (MultiBalanceReport (colspans, rows, totalsrow)) =
+  MultiBalanceReport (colspans, map mbrRowNegate rows, mbrTotalsRowNegate totalsrow)
+  where
+    mbrRowNegate (acct,shortacct,indent,amts,tot,avg) = (acct,shortacct,indent,map negate amts,-tot,-avg)
+    mbrTotalsRowNegate (amts,tot,avg) = (map negate amts,-tot,-avg)
+
+-- | Figure out the overall date span of a multicolumn balance report.
+multiBalanceReportSpan :: MultiBalanceReport -> DateSpan
+multiBalanceReportSpan (MultiBalanceReport ([], _, _))       = DateSpan Nothing Nothing
+multiBalanceReportSpan (MultiBalanceReport (colspans, _, _)) = DateSpan (spanStart $ head colspans) (spanEnd $ last colspans)
+
+-- | Generates a simple non-columnar BalanceReport, but using multiBalanceReport, 
+-- in order to support --historical. Does not support tree-mode boring parent eliding. 
+-- If the normalbalance_ option is set, it adjusts the sorting and sign of amounts 
+-- (see ReportOpts and CompoundBalanceCommand).
+balanceReportFromMultiBalanceReport :: ReportOpts -> Query -> Journal -> BalanceReport
+balanceReportFromMultiBalanceReport opts q j = (rows', total)
+  where
+    MultiBalanceReport (_, rows, (totals, _, _)) = multiBalanceReport opts q j
+    rows' = [(a
+             ,if flat_ opts then a else a'   -- BalanceReport expects full account name here with --flat
+             ,if tree_ opts then d-1 else 0  -- BalanceReport uses 0-based account depths
+             , headDef nullmixedamt amts     -- 0 columns is illegal, should not happen, return zeroes if it does
+             ) | (a,a',d, amts, _, _) <- rows]
+    total = headDef nullmixedamt totals
+
 
 tests_multiBalanceReport =
   let
@@ -281,6 +332,22 @@ tests_multiBalanceReport =
       Mixed [usd0])
   ]
 
+-- common rendering helper, XXX here for now
+
+tableAsText :: ReportOpts -> (a -> String) -> Table String String a -> String
+tableAsText (ReportOpts{pretty_tables_ = pretty}) showcell =
+  unlines
+  . trimborder
+  . lines
+  . render pretty id id showcell
+  . align
+  where
+    trimborder = drop 1 . init . map (drop 1 . init)
+    align (Table l t d) = Table l' t d
+      where
+        acctswidth = maximum' $ map strWidth (headerContents l)
+        l'         = padRightWide acctswidth <$> l
+
 tests_Hledger_Reports_MultiBalanceReport :: Test
 tests_Hledger_Reports_MultiBalanceReport = TestList
   tests_multiBalanceReport
