@@ -19,12 +19,13 @@ module Hledger.Reports.ReportOptions (
   simplifyStatuses,
   whichDateFromOpts,
   journalSelectingAmountFromOpts,
+  intervalFromRawOpts,
   queryFromOpts,
   queryFromOptsOnly,
   queryOptsFromOpts,
   transactionDateFn,
   postingDateFn,
-  reportStartEndDates,
+  reportSpan,
   reportStartDate,
   reportEndDate,
   specifiedStartEndDates,
@@ -35,10 +36,8 @@ module Hledger.Reports.ReportOptions (
 )
 where
 
+import Control.Applicative ((<|>))
 import Data.Data (Data)
-#if !MIN_VERSION_base(4,8,0)
-import Data.Functor.Compat ((<$>))
-#endif
 import Data.List
 import Data.Maybe
 import qualified Data.Text as T
@@ -102,13 +101,17 @@ data ReportOpts = ReportOpts {
     ,value_          :: Bool
     ,pretty_tables_  :: Bool
     ,sort_amount_    :: Bool
-    ,normalbalance_  :: Maybe NormalBalance
-      -- ^ when running a balance report on accounts of the same normal balance type,
-      -- eg in the income section of an income statement, this helps --sort-amount know
-      -- how to sort negative numbers.
+    ,invert_         :: Bool  -- ^ if true, flip all amount signs in reports
+    ,normalbalance_  :: Maybe NormalSign
+      -- ^ This can be set when running balance reports on a set of accounts
+      --   with the same normal balance type (eg all assets, or all incomes).
+      -- - It helps --sort-amount know how to sort negative numbers
+      --   (eg in the income section of an income statement) 
+      -- - It helps compound balance report commands (is, bs etc.) do  
+      --   sign normalisation, converting normally negative subreports to 
+      --   normally positive for a more conventional display.   
     ,color_          :: Bool
     ,forecast_       :: Bool
-    ,auto_           :: Bool
  } deriving (Show, Data, Typeable)
 
 instance Default ReportOpts where def = defreportopts
@@ -169,10 +172,10 @@ rawOptsToReportOpts rawopts = checkReportOpts <$> do
     ,no_total_    = boolopt "no-total" rawopts'
     ,value_       = boolopt "value" rawopts'
     ,sort_amount_ = boolopt "sort-amount" rawopts'
+    ,invert_      = boolopt "invert" rawopts'
     ,pretty_tables_ = boolopt "pretty-tables" rawopts'
     ,color_       = color
     ,forecast_    = boolopt "forecast" rawopts'
-    ,auto_        = boolopt "auto" rawopts'
     }
 
 -- | Do extra validation of raw option values, raising an error if there's a problem.
@@ -402,24 +405,27 @@ tests_queryOptsFromOpts = [
                                                              })
  ]
 
--- | The effective report start/end dates are the dates specified by options or queries,
--- otherwise the earliest/latest transaction or posting date in the journal,
--- otherwise (for an empty journal) nothing.
+-- | The effective report span is the start and end dates specified by
+-- options or queries, or otherwise the earliest and latest transaction or 
+-- posting dates in the journal. If no dates are specified by options/queries
+-- and the journal is empty, returns the null date span.
 -- Needs IO to parse smart dates in options/queries.
-reportStartEndDates :: Journal -> ReportOpts -> IO (Maybe (Day,Day))
-reportStartEndDates j ropts = do
-  (mspecifiedstartdate, mspecifiedenddate) <- specifiedStartEndDates ropts
-  return $
-    case journalDateSpan False j of  -- don't bother with secondary dates
-      DateSpan (Just journalstartdate) (Just journalenddate) ->
-        Just (fromMaybe journalstartdate mspecifiedstartdate, fromMaybe journalenddate mspecifiedenddate)
-      _ -> Nothing
+reportSpan :: Journal -> ReportOpts -> IO DateSpan
+reportSpan j ropts = do
+  (mspecifiedstartdate, mspecifiedenddate) <-
+    dbg2 "specifieddates" <$> specifiedStartEndDates ropts
+  let
+    DateSpan mjournalstartdate mjournalenddate =
+      dbg2 "journalspan" $ journalDateSpan False j  -- ignore secondary dates
+    mstartdate = mspecifiedstartdate <|> mjournalstartdate
+    menddate   = mspecifiedenddate   <|> mjournalenddate
+  return $ dbg1 "reportspan" $ DateSpan mstartdate menddate
 
 reportStartDate :: Journal -> ReportOpts -> IO (Maybe Day)
-reportStartDate j ropts = (fst <$>) <$> reportStartEndDates j ropts
+reportStartDate j ropts = spanStart <$> reportSpan j ropts
 
 reportEndDate :: Journal -> ReportOpts -> IO (Maybe Day)
-reportEndDate j ropts = (snd <$>) <$> reportStartEndDates j ropts
+reportEndDate j ropts = spanEnd <$> reportSpan j ropts
 
 -- | The specified report start/end dates are the dates specified by options or queries, if any.
 -- Needs IO to parse smart dates in options/queries.
