@@ -33,7 +33,9 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           -- the rest need to be done in each module I think
                           )
 where
-import Control.Monad (liftM)
+import Test.HUnit
+
+import Control.Monad (liftM, when)
 -- import Data.Char
 import Data.Default
 import Data.List
@@ -150,30 +152,28 @@ firstJust ms = case dropWhile (==Nothing) ms of
     [] -> Nothing
     (md:_) -> md
 
--- | Read a file in universal newline mode, handling any of the usual line ending conventions.
-readFile' :: FilePath -> IO Text
-readFile' name =  do
-  h <- openFile name ReadMode
-  hSetNewlineMode h universalNewlineMode
-  T.hGetContents h
+-- | Read text from a file, 
+-- handling any of the usual line ending conventions,
+-- using the system locale's text encoding,
+-- ignoring any utf8 BOM prefix (as seen in paypal's 2018 CSV, eg) if that encoding is utf8. 
+readFilePortably :: FilePath -> IO Text
+readFilePortably f =  openFile f ReadMode >>= readHandlePortably
 
--- | Read a file in universal newline mode, handling any of the usual line ending conventions.
-readFileAnyLineEnding :: FilePath -> IO Text
-readFileAnyLineEnding path =  do
-  h <- openFile path ReadMode
-  hSetNewlineMode h universalNewlineMode
-  T.hGetContents h
-
--- | Read the given file, or standard input if the path is "-", using
--- universal newline mode.
-readFileOrStdinAnyLineEnding :: String -> IO Text
-readFileOrStdinAnyLineEnding f = do
-  h <- fileHandle f
-  hSetNewlineMode h universalNewlineMode
-  T.hGetContents h
+-- | Like readFilePortably, but read from standard input if the path is "-". 
+readFileOrStdinPortably :: String -> IO Text
+readFileOrStdinPortably f = openFileOrStdin f ReadMode >>= readHandlePortably
   where
-    fileHandle "-" = return stdin
-    fileHandle f = openFile f ReadMode
+    openFileOrStdin :: String -> IOMode -> IO Handle
+    openFileOrStdin "-" _ = return stdin
+    openFileOrStdin f m   = openFile f m
+
+readHandlePortably :: Handle -> IO Text
+readHandlePortably h = do
+  hSetNewlineMode h universalNewlineMode
+  menc <- hGetEncoding h
+  when (fmap show menc == Just "UTF-8") $  -- XXX no Eq instance, rely on Show
+    hSetEncoding h utf8_bom
+  T.hGetContents h
 
 -- | Total version of maximum, for integral types, giving 0 for an empty list.
 maximum' :: Integral a => [a] -> a
@@ -214,3 +214,8 @@ sequence' ms = do
 {-# INLINABLE mapM' #-}
 mapM' :: Monad f => (a -> f b) -> [a] -> f [b]
 mapM' f = sequence' . map f
+
+tests_Hledger_Utils :: Test
+tests_Hledger_Utils = TestList [
+    tests_Hledger_Utils_Text
+    ]
\ No newline at end of file
